SystemOrganization addCategory: #'RBC-VSA-Core'!
SystemOrganization addCategory: #'RBC-VSA-CPS'!
SystemOrganization addCategory: #'RBC-VSA-Heater'!
SystemOrganization addCategory: #'RBC-VSA-Portfolio'!

TestCase subclass: #AccountTest
	instanceVariableNames: 'myAccount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!AccountTest commentStamp: 'rbc 8/20/2011 09:30' prior: 0!
Test the Account class.!

!AccountTest methodsFor: 'setup' stamp: 'rbc 7/10/2011 12:28'!
setUp
	"Setup account VSA test"
	myAccount := Account new! !

!AccountTest methodsFor: 'testing' stamp: 'rbc 7/10/2011 12:43'!
testName
	"Test setting and retrieving the account name"
	| n |
	n := 'ABC Account'.
	myAccount accountName: n.
	self assert: (myAccount accountName = n)! !

!AccountTest methodsFor: 'testing' stamp: 'rbc 7/10/2011 12:43'!
testNumber
	"Test setting and retrieving the account number"
	| n |
	n := '001003005'.
	myAccount accountNumber: n.
	self assert: (myAccount accountNumber = n)! !

TestCase subclass: #CashTest
	instanceVariableNames: 'myCash'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!CashTest commentStamp: 'rbc 9/3/2011 12:35' prior: 0!
Testing for the Cash class.
!

!CashTest methodsFor: 'setup' stamp: 'rbc 9/3/2011 12:07'!
setUp
	"Setup cash VSA test"
	myCash := Cash new! !

!CashTest methodsFor: 'testing' stamp: 'rbc 9/3/2011 12:09'!
testDeposit
	"Test deposits"
	myCash initialBalance: 50.
	myCash deposit: 50.
	self assert: (myCash balance = 100)! !

!CashTest methodsFor: 'testing' stamp: 'rbc 9/3/2011 12:12'!
testDescription
	"Test description"
	| d |
	d := 'My cash'.
	myCash description: d.
	self assert: (myCash description = d)! !

!CashTest methodsFor: 'testing' stamp: 'rbc 9/3/2011 12:21'!
testOverdraw
	"Test overdrawn posistion"
	| r |
	myCash initialBalance: 50.
	r := myCash withdraw: 100.
	self assert: ( r = nil)! !

!CashTest methodsFor: 'testing' stamp: 'rbc 9/3/2011 12:07'!
testPrice
	"Test setting the price"
	myCash initialBalance: 50.
	self assert: (myCash balance = 50)! !

!CashTest methodsFor: 'testing' stamp: 'rbc 9/3/2011 12:33'!
testWithdraw
	"Test withdrawal"
	| r |
	myCash initialBalance: 50.
	r := myCash withdraw: 10.
	self assert: (myCash balance = 40).
	self assert: (r = 10)! !

TestCase subclass: #CyberPhysicalSystemTest
	instanceVariableNames: 'myCPS'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-CPS'!

!CyberPhysicalSystemTest commentStamp: 'rbc 9/20/2011 20:13' prior: 0!
Testing for the cyber physical system.
!

!CyberPhysicalSystemTest methodsFor: 'setup' stamp: 'rbc 9/12/2011 22:19'!
setUp
	"Setup test environment"
	myCPS := CyberPhysicalSystem new

! !

!CyberPhysicalSystemTest methodsFor: 'testing' stamp: 'rbc 9/20/2011 19:57'!
testGets
	"Test out get methods"
	| la lo el f p c |
	"Test basic VSA functionality"
	myCPS actuality: 0.
	myCPS capability: 2.
	myCPS potentiality: 4.
	myCPS alertVariance: 0.
	"Cause an alert to be generated"
	myCPS mark: VSAAlert.
	"Check the results"
	self assert: (myCPS actuality = 0).
	self assert: (myCPS capability = 2).
	self assert: (myCPS potentiality = 4).
	self assert: (myCPS alert isMemberOf: VSAAlert).
	self assert: (myCPS myExecutive isMemberOf: VSAExecutive).
	self assert: (myCPS myPlanner isMemberOf: VSAPlanner).
	self assert: (myCPS myManager isMemberOf: VSAManager).
	self assert: (myCPS costCenter = false).
	self assert: (myCPS myManager costCenter = false).
	"Flip the VSA into being a cost center"
	myCPS costCenter: true.
	self assert: (myCPS costCenter = true).
	self assert: (myCPS myManager costCenter = true).
	"Test geospacial index"
	la := '37'.
	lo := '-122'.
	el := '0'.
	myCPS myGSI latitude: la.
	myCPS myGSI longitude: lo.
	myCPS myGSI elevation: el.
	self assert: (myCPS myGSI latitude = la).
	self assert: (myCPS myGSI longitude = lo).
	self assert: (myCPS myGSI elevation = el).
	"Test sociospacial index"
	f := 'Unified Combatant Command'.
	p := 'J6'.
	c := 'Pacific Command'.
	myCPS mySSI federation: f.
	myCPS mySSI productionUnit: p.
	myCPS mySSI commandActor: c.
	self assert: (myCPS mySSI federation = f).
	self assert: (myCPS mySSI productionUnit = p).
	self assert: (myCPS mySSI commandActor = c)! !

TestCase subclass: #CyberspacialObjectTest
	instanceVariableNames: 'myCSO'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-CPS'!

!CyberspacialObjectTest commentStamp: 'rbc 9/20/2011 20:13' prior: 0!
Testing for the cyberspatial object.!

!CyberspacialObjectTest methodsFor: 'setup' stamp: 'rbc 9/20/2011 20:05'!
setUp
	"Setup test environment"
	myCSO := CyberspatialObject new

! !

!CyberspacialObjectTest methodsFor: 'testing' stamp: 'rbc 9/20/2011 20:05'!
testGets
	"Test out get methods"
	| la lo el f p c gn sn sp |
	"Test basic VSA functionality"
	myCSO actuality: 0.
	myCSO capability: 2.
	myCSO potentiality: 4.
	myCSO alertVariance: 0.
	"Cause an alert to be generated"
	myCSO mark: VSAAlert.
	"Check the results"
	self assert: (myCSO actuality = 0).
	self assert: (myCSO capability = 2).
	self assert: (myCSO potentiality = 4).
	self assert: (myCSO alert isMemberOf: VSAAlert).
	self assert: (myCSO myExecutive isMemberOf: VSAExecutive).
	self assert: (myCSO myPlanner isMemberOf: VSAPlanner).
	self assert: (myCSO myManager isMemberOf: VSAManager).
	self assert: (myCSO costCenter = false).
	self assert: (myCSO myManager costCenter = false).
	"Test geospacial index"
	la := '37'.
	lo := '-122'.
	el := '0'.
	myCSO myGSI latitude: la.
	myCSO myGSI longitude: lo.
	myCSO myGSI elevation: el.
	self assert: (myCSO myGSI latitude = la).
	self assert: (myCSO myGSI longitude = lo).
	self assert: (myCSO myGSI elevation = el).
	"Test sociospacial index"
	f := 'Unified Combatant Command'.
	p := 'J6'.
	c := 'Pacific Command'.
	myCSO mySSI federation: f.
	myCSO mySSI productionUnit: p.
	myCSO mySSI commandActor: c.
	self assert: (myCSO mySSI federation = f).
	self assert: (myCSO mySSI productionUnit = p).
	self assert: (myCSO mySSI commandActor = c).
	"Test infospacial index"
	gn := '10.0.0.1'.
	sn := '255.255.255.0'.
	sp := '53'.
	myCSO myISI globalNet: gn.
	myCSO myISI subNet: sn.
	myCSO myISI servicePoint: sp.
	self assert: (myCSO myISI globalNet = gn).
	self assert: (myCSO myISI subNet = sn).
	self assert: (myCSO myISI servicePoint = sp)! !

TestCase subclass: #GeospacialIndexTest
	instanceVariableNames: 'myGSI'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-CPS'!

!GeospacialIndexTest commentStamp: 'rbc 9/16/2011 13:00' prior: 0!
Testing for the geospacial index.
!

!GeospacialIndexTest methodsFor: 'setup' stamp: 'rbc 9/12/2011 21:45'!
setUp
	"Setup test environment"
	myGSI := GeospacialIndex new

! !

!GeospacialIndexTest methodsFor: 'testing' stamp: 'rbc 9/12/2011 21:52'!
testGets
	"Test out get methods"
	| la lo el |
	la := '37'.
	lo := '-122'.
	el := '0'.
	myGSI latitude: la.
	myGSI longitude: lo.
	myGSI elevation: el.
	self assert: (myGSI latitude = la).
	self assert: (myGSI longitude = lo).
	self assert: (myGSI elevation = el)! !

TestCase subclass: #HeaterTest
	instanceVariableNames: 'myVSA'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Heater'!

!HeaterTest commentStamp: 'rbc 7/31/2011 16:47' prior: 0!
Unit testing for heating a building.
!

!HeaterTest methodsFor: 'setup' stamp: 'rbc 7/31/2011 16:12'!
setUp
	"Setup test environment"
	myVSA := VSA new! !

!HeaterTest methodsFor: 'testing' stamp: 'rbc 7/31/2011 18:02'!
testHeater
	"Test heating a building."
	| lobby office |
	myVSA myExecutive vName: 'Building 1'.
	"Setup and test the lobby."
	myVSA myManager createOp: HeaterOperation.
	lobby := myVSA myManager getOp: 1.
	lobby myExecutive vName: 'Lobby'.
	lobby desiredTemperature: 66.
	lobby transferHeat.
	lobby heatRoom.
	self assert: (lobby myRoom temperature = 66).
	"Setup and test the office."
	myVSA myManager createOp: HeaterOperation.
	office := myVSA myManager getOp: 2.
	office myExecutive vName: 'Office'.
	office desiredTemperature: 68.
	office transferHeat.
	office heatRoom.
	self assert: (office myRoom temperature = 68).
	"Test aggregate minimum, mean and maximum temperatures for the building."
	self assert: (myVSA myManager actualityMin = 66).
	self assert: (myVSA myManager actualityMean = 67).
	self assert: (myVSA myManager actualityMax = 68).

! !

TestCase subclass: #InfospacialIndexTest
	instanceVariableNames: 'myISI'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-CPS'!

!InfospacialIndexTest commentStamp: 'rbc 9/12/2011 22:35' prior: 0!
Testing for the infospacial index.
!

!InfospacialIndexTest methodsFor: 'setup' stamp: 'rbc 9/12/2011 21:54'!
setUp
	"Setup test environment"
	myISI := InfospacialIndex new

! !

!InfospacialIndexTest methodsFor: 'testing' stamp: 'rbc 9/12/2011 21:59'!
testGets
	"Test out get methods"
	| gn sn sp |
	gn := '10.0.0.1'.
	sn := '255.255.255.0'.
	sp := '53'.
	myISI globalNet: gn.
	myISI subNet: sn.
	myISI servicePoint: sp.
	self assert: (myISI globalNet = gn).
	self assert: (myISI subNet = sn).
	self assert: (myISI servicePoint = sp)! !

TestCase subclass: #InstitutionTest
	instanceVariableNames: 'myInstitution'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!InstitutionTest commentStamp: 'rbc 8/20/2011 09:32' prior: 0!
Test the Institution class.!

!InstitutionTest methodsFor: 'setup' stamp: 'rbc 7/10/2011 12:20'!
setUp
	"Setup institution VSA test"
	myInstitution := Institution new.! !

!InstitutionTest methodsFor: 'testing' stamp: 'rbc 7/10/2011 12:44'!
testName
	"Test setting and retrieving the institution name"
	| n |
	n := 'ABC Institution'.
	myInstitution institutionName: n.
	self assert: (myInstitution institutionName = n)! !

TestCase subclass: #PortfolioTest
	instanceVariableNames: 'myInstitution myAccount anotherAccount myStock myCall anotherStock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!PortfolioTest commentStamp: 'rbc 8/20/2011 09:32' prior: 0!
Test the portfolio.

Copyright 2011 Robert Bruce Carleton. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY ROBERT BRUCE CARLETON ''AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ROBERT BRUCE CARLETON OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those of the
authors and should not be interpreted as representing official policies, either expressed
or implied, of Robert Bruce Carleton.
!

!PortfolioTest methodsFor: 'setup' stamp: 'rbc 7/10/2011 13:40'!
setUp
	"Set up a end to end portfolio test"
	myInstitution := Institution new
! !

!PortfolioTest methodsFor: 'testing' stamp: 'rbc 7/10/2011 17:16'!
testRollup
	"Test rolling up an account and institution"
	myInstitution myManager createOp: Account.
	myAccount := myInstitution myManager getOp: 1.
	myAccount myManager createOp: Stock.
	myStock := myAccount myManager getOp: 1.
	myStock stockPrice: 50.
	myStock numberShares: 100.
	myAccount update.
	self assert: (myAccount actuality = 5000).
	myAccount myManager createOp: StockOptionCall.
	myCall := myAccount myManager getOp: 2.
	myCall marketPrice: 2.
	myCall numberContracts: 2.
	myAccount update.
	self assert: (myAccount actuality = 5400).
	myInstitution update.
	self assert: (myInstitution actuality = 5400).
	myInstitution myManager createOp: Account.
	anotherAccount := myInstitution myManager getOp: 2.
	anotherAccount myManager createOp: Stock.
	anotherStock := anotherAccount myManager getOp: 1.
	anotherStock stockPrice: 10.
	anotherStock numberShares: 100.
	anotherAccount update.
	self assert: (anotherAccount actuality = 1000).
	myInstitution update.
	self assert: (myInstitution actuality = 6400).

! !

TestCase subclass: #SociospacialIndexTest
	instanceVariableNames: 'mySSI'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-CPS'!

!SociospacialIndexTest commentStamp: 'rbc 9/12/2011 22:36' prior: 0!
Testing for the sociospacial index.!

!SociospacialIndexTest methodsFor: 'setup' stamp: 'rbc 9/12/2011 22:01'!
setUp
	"Setup test environment"
	mySSI := SociospacialIndex new

! !

!SociospacialIndexTest methodsFor: 'testing' stamp: 'rbc 9/12/2011 22:18'!
testGets
	"Test out get methods"
	| f p c |
	f := 'Unified Combatant Command'.
	p := 'J6'.
	c := 'Pacific Command'.
	mySSI federation: f.
	mySSI productionUnit: p.
	mySSI commandActor: c.
	self assert: (mySSI federation = f).
	self assert: (mySSI productionUnit = p).
	self assert: (mySSI commandActor = c)! !

TestCase subclass: #StockCreditRatioTest
	instanceVariableNames: 'myRatio'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockCreditRatioTest commentStamp: 'rbc 8/14/2011 14:56' prior: 0!
Test the StockCreditRatio class.
!

!StockCreditRatioTest methodsFor: 'setup' stamp: 'rbc 8/14/2011 11:40'!
setUp
	"Setup stock credit ratio test"
	myRatio := StockCreditRatio new
! !

!StockCreditRatioTest methodsFor: 'testing' stamp: 'rbc 8/14/2011 13:27'!
testCashFlowCoverSeniorCharges
	"Test returning cash flow coverage of senior charges"
	myRatio cashFlowAfterTaxes: 20070.
	myRatio seniorCharges: 2994.
	self assert: (myRatio cashFlowCoverSeniorCharges = (3844/499))! !

!StockCreditRatioTest methodsFor: 'testing' stamp: 'rbc 8/14/2011 14:21'!
testCashFlowTotalCapital
	"Test returning cash flow to total capital"
	myRatio cashFlowAfterTaxes: 99.
	myRatio taxAdjustedInterest: 1.
	myRatio tangibleAssets: 400.
	myRatio accruedPayables: 5.
	self assert: (myRatio cashFlowTotalCapital = (20/79))! !

!StockCreditRatioTest methodsFor: 'testing' stamp: 'rbc 8/14/2011 14:54'!
testCashRatio
	"Test returning the cash ratio."
	myRatio cashItems: 100.
	myRatio currentLiabilities: 75.
	self assert: (myRatio cashRatio = (4/3))! !

!StockCreditRatioTest methodsFor: 'testing' stamp: 'rbc 8/14/2011 15:05'!
testCoverageSeniorCharges
	"Test returning coverage of senior charges"
	myRatio pretaxEarned: 100.
	myRatio seniorCharges: 75.
	self assert: (myRatio coverageSeniorCharges = (4/3))! !

!StockCreditRatioTest methodsFor: 'testing' stamp: 'rbc 8/14/2011 15:10'!
testCurrentRatio
	"Test returning the current ratio."
	myRatio currentAssets: 100.
	myRatio currentLiabilities: 99.
	self assert: (myRatio currentRatio = (100/99))! !

!StockCreditRatioTest methodsFor: 'testing' stamp: 'rbc 8/14/2011 15:22'!
testDefensiveIntervalDays
	"Test returning the defensive interval in days."
	myRatio cash: 100.
	myRatio receivables: 80.
	myRatio expenses: 140.
	myRatio depreciation: 10.
	myRatio nonCashCharges: 5.
	self assert: (myRatio defensiveIntervalDays = (2628/5))! !

!StockCreditRatioTest methodsFor: 'testing' stamp: 'rbc 8/14/2011 15:34'!
testEquityRatio
	"Test returning the equity ratio."
	myRatio commonEquityBook: 10.
	myRatio tangibleAssets: 6.
	myRatio accruedPayables: 4.
	self assert: (myRatio equityRatio = 5)! !

!StockCreditRatioTest methodsFor: 'testing' stamp: 'rbc 8/14/2011 15:43'!
testEquityRatioMarket
	"Returns equity ratio at market"
	myRatio commonEquityMarket: 100.
	myRatio tangibleAssets: 90.
	myRatio accruedPayables: 10.
	self assert: (myRatio equityRatioMarket = (5/4))! !

!StockCreditRatioTest methodsFor: 'testing' stamp: 'rbc 1/31/2012 18:54'!
testGets
	"Test setters and getters"

	myRatio pretaxEarned: 100.
	self assert: (myRatio pretaxEarned = 100).
	
	myRatio currentAssets: 100.
	self assert: (myRatio currentAssets = 100).
	
	myRatio currentLiabilities: 100.
	self assert: (myRatio currentLiabilities = 100).
	
	myRatio rent: 100.
	self assert: (myRatio rent = 100).
	
	myRatio currentMaturities: 100.
	self assert: (myRatio currentMaturities = 100).
	
	myRatio receivables: 100.
	self assert: (myRatio receivables = 100).
	
	myRatio cash: 100.
	self assert: (myRatio cash = 100).
	
	myRatio rentals: 100.
	self assert: (myRatio rentals = 100).
	
	myRatio commonEquityBook: 100.
	self assert: (myRatio commonEquityBook = 100).
	
	myRatio commonEquityMarket: 100.
	self assert: (myRatio commonEquityMarket = 100).
	
	myRatio tangibleAssets: 100.
	self assert: (myRatio tangibleAssets = 100).
	
	myRatio nonCashCharges: 100.
	self assert: (myRatio nonCashCharges = 100).
	
	myRatio sinkingFundPayments: 100.
	self assert: (myRatio sinkingFundPayments = 100).
	
	myRatio taxAdjustedInterest: 100.
	self assert: (myRatio taxAdjustedInterest = 100).
	
	myRatio seniorCharges: 100.
	self assert: (myRatio seniorCharges = 100).
	
	myRatio interest: 100.
	self assert: (myRatio interest = 100).
	
	myRatio depreciation: 100.
	self assert: (myRatio depreciation = 100).
	
	myRatio cashItems: 100.
	self assert: (myRatio cashItems = 100).
	
	myRatio accruedPayables: 100.
	self assert: (myRatio accruedPayables = 100).
	
	myRatio expenses: 100.
	self assert: (myRatio expenses = 100).
	
	myRatio cashFlowAfterTaxes: 100.
	self assert: (myRatio cashFlowAfterTaxes = 100).
	
! !

!StockCreditRatioTest methodsFor: 'testing' stamp: 'rbc 8/14/2011 15:49'!
testQuickRatio
	"Test returning the quick ratio."
	myRatio currentAssets: 20.
	myRatio inventories: 5.
	myRatio liabilities: 5.
	self assert: (myRatio quickRatio = 3)! !

!StockCreditRatioTest methodsFor: 'testing' stamp: 'rbc 8/14/2011 16:01'!
testTotalDebtServiceCoverage
	"Test returning total debt service coverage."
	myRatio cashFlowAfterTaxes: 70.
	myRatio rentals: 5.
	myRatio taxAdjustedInterest: 5.
	myRatio interest: 2.
	myRatio rent: 10.
	myRatio currentMaturities: 4.
	myRatio sinkingFundPayments: 4.
	self assert: (myRatio totalDebtServiceCoverage = 4)! !

TestCase subclass: #StockGrowthRatioTest
	instanceVariableNames: 'myRatio'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockGrowthRatioTest commentStamp: 'rbc 8/16/2011 21:59' prior: 0!
Test the StockGrowthRatio class.!

!StockGrowthRatioTest methodsFor: 'setup' stamp: 'rbc 8/16/2011 20:58'!
setUp
	"Setup stock growth ratio test"
	myRatio := StockGrowthRatio new
! !

!StockGrowthRatioTest methodsFor: 'testing' stamp: 'rbc 1/31/2012 19:09'!
testGets
	"Test setters and getters"

	myRatio epsInFinalPeriod: 0.03.
	self assert: (myRatio epsInFinalPeriod = 0.03).
	
	myRatio baseSales: 5.
	self assert: (myRatio baseSales = 5).
	
	myRatio epsInBasePeriod: 0.06.
	self assert: (myRatio epsInBasePeriod = 0.06).
	
	myRatio finalSales: 10.
	self assert: (myRatio finalSales = 10).
	
	myRatio netEarnedBase: 5.
	self assert: (myRatio netEarnedBase = 5).
	
	myRatio netEarnedFinal: 10.
	self assert: (myRatio netEarnedFinal = 10)! !

!StockGrowthRatioTest methodsFor: 'testing' stamp: 'rbc 8/16/2011 21:00'!
testGrowthInEPS
	"Test returning growth in earnings per share."
	myRatio epsInFinalPeriod: 0.03.
	myRatio epsInBasePeriod: 0.06.
	self assert: (myRatio growthInEPS = 0.5)! !

!StockGrowthRatioTest methodsFor: 'testing' stamp: 'rbc 8/16/2011 21:08'!
testGrowthInSales
	"Test returning growth in sales."
	myRatio finalSales: 10.
	myRatio baseSales: 5.
	self assert: (myRatio growthInSales = 2)! !

!StockGrowthRatioTest methodsFor: 'testing' stamp: 'rbc 8/16/2011 21:15'!
testGrowthInTotalReturn
	"Get growth in total return"
	myRatio netEarnedFinal: 10.
	myRatio netEarnedBase: 5.
	self assert: (myRatio growthInTotalReturn = 2)! !

TestCase subclass: #StockMiscRatioTest
	instanceVariableNames: 'myRatio'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockMiscRatioTest commentStamp: 'rbc 8/16/2011 21:59' prior: 0!
Test the StockMiscRatio class.!

!StockMiscRatioTest methodsFor: 'setup' stamp: 'rbc 8/16/2011 21:20'!
setUp
	"Setup other stock ratios test"
	myRatio := StockMiscRatio new
! !

!StockMiscRatioTest methodsFor: 'testing' stamp: 'rbc 8/16/2011 21:23'!
testAccountsReceivableTurnover
	"Test returning accounts receivable turnover."
	myRatio sales: 8.
	myRatio receivables:4.
	self assert: (myRatio accountsReceivableTurnover = 2)! !

!StockMiscRatioTest methodsFor: 'testing' stamp: 'rbc 8/16/2011 21:26'!
testDepreciationToGrossPlant
	"Test returning depreciation to gross plant."
	myRatio depreciation: 2.
	myRatio grossPlant: 10.
	self assert: (myRatio depreciationToGrossPlant = (1/5))! !

!StockMiscRatioTest methodsFor: 'testing' stamp: 'rbc 8/16/2011 21:28'!
testDepreciationToSales
	"Test returning depreciation to sales."
	myRatio depreciation: 2.
	myRatio sales: 8.
	self assert: (myRatio depreciationToSales = (1/4))! !

!StockMiscRatioTest methodsFor: 'testing' stamp: 'rbc 1/31/2012 19:40'!
testGets
	"Test setters and getters"

	myRatio sales: 8.
	self assert: (myRatio sales = 8).

	myRatio receivables: 4.
	self assert: (myRatio receivables = 4).

	myRatio depreciation: 2.
	self assert: (myRatio depreciation = 2).

	myRatio grossPlant: 10.
	self assert: (myRatio grossPlant = 10).

	myRatio costOfGoodsSold: 2.
	self assert: (myRatio costOfGoodsSold = 2).

	myRatio inventory: 1.
	self assert: (myRatio inventory = 1).

! !

!StockMiscRatioTest methodsFor: 'testing' stamp: 'rbc 8/16/2011 21:34'!
testInventoryTurnover
	"Test returning inventory turnover."
	myRatio costOfGoodsSold: 2.
	myRatio inventory: 1.
	self assert: (myRatio inventoryTurnover = 2)
	! !

TestCase subclass: #StockOptionTest
	instanceVariableNames: 'myCall myPut'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockOptionTest commentStamp: 'rbc 8/20/2011 09:28' prior: 0!
Test the StockOption class.!

!StockOptionTest methodsFor: 'setup' stamp: 'rbc 7/5/2011 21:22'!
setUp
	"Setup stock VSA test"
	myCall := StockOptionCall new.
	myPut := StockOptionPut new.! !

!StockOptionTest methodsFor: 'testing' stamp: 'rbc 7/10/2011 14:28'!
testActuality
	"Test getting the actuality"
	myCall marketPrice: 2.
	myCall numberContracts: 1.
	self assert: (myCall actuality = 200)! !

!StockOptionTest methodsFor: 'testing' stamp: 'rbc 7/5/2011 21:40'!
testBreakEven
	"Test calculation of the option break even price"
	myCall strikePrice: 50.
	myCall premium: 2.
	self assert: (myCall breakEven = 52).
	myPut strikePrice: 50.
	myPut premium: 2.
	self assert: (myPut breakEven = 48)! !

!StockOptionTest methodsFor: 'testing' stamp: 'rbc 7/5/2011 21:22'!
testDescription
	"Test setting the stock description"
	| d |
	d := 'ABC Company'.
	myCall description: d.
	self assert: (myCall description = d)! !

!StockOptionTest methodsFor: 'testing' stamp: 'rbc 7/10/2011 17:29'!
testExpiration
	"Test setting the option expiration"
	| e |
	e := Date dateAndTimeNow.
	myCall expiration: e.
	self assert: (myCall expiration = e)! !

!StockOptionTest methodsFor: 'testing' stamp: 'rbc 1/31/2012 18:25'!
testGets
	"Test setters and getters"

	"Striking price"
	myCall strikePrice: 50.
	self assert: (myCall strikePrice = 50).

	"Number of contracts"
	myCall numberContracts: 1.
	self assert: (myCall numberContracts = 1).

! !

!StockOptionTest methodsFor: 'testing' stamp: 'rbc 1/31/2012 18:15'!
testIntrinsicValue
	"Test calculation of the option intrinsic value"
	
	"Call"
	myCall stockPrice: 53.
	myCall strikePrice: 50.
	self assert: (myCall intrinsicValue = 3).
	"Put"
	myPut stockPrice: 48.
	myPut strikePrice: 50.
	self assert: (myPut intrinsicValue = 2)! !

!StockOptionTest methodsFor: 'testing' stamp: 'rbc 7/6/2011 02:00'!
testMarketPrice
	"Test setting the market price"
	| m |
	m := 50.
	myCall marketPrice: m.
	self assert: (myCall marketPrice = m).
	! !

!StockOptionTest methodsFor: 'testing' stamp: 'rbc 7/5/2011 21:23'!
testNumberShares
	"Test setting the number of shares"
	myCall numberShares: 100.
	self assert: (myCall numberShares = 100)! !

!StockOptionTest methodsFor: 'testing' stamp: 'rbc 7/5/2011 21:30'!
testPremium
	"Test setting the option premium"
	| p |
	p := 50.
	myCall premium: p.
	self assert: (myCall premium = p)! !

!StockOptionTest methodsFor: 'testing' stamp: 'rbc 7/7/2011 08:23'!
testPrices
	"Test setting prices"
	| cm cs pm ps |
	cm := 2.
	cs := 51.
	pm := 2.
	ps := 55.
	myCall marketPrice: cm.
	myCall stockPrice: cs.
	myPut marketPrice: pm.
	myPut stockPrice: ps.
	self assert: (myCall marketPrice = cm).
	self assert: (myCall stockPrice = cs).
	self assert: (myPut marketPrice = pm).
	self assert: (myPut stockPrice = ps)! !

!StockOptionTest methodsFor: 'testing' stamp: 'rbc 7/5/2011 21:23'!
testTicker
	"Test setting the ticker"
	| t |
	t := 'ABC'.
	myCall ticker: t.
	self assert: (myCall ticker = t)! !

!StockOptionTest methodsFor: 'testing' stamp: 'rbc 1/31/2012 18:20'!
testTimeValue
	"Test calculation of the option time value"
	
	"Call"
	myCall stockPrice: 53.
	myCall strikePrice: 50.
	self assert: (myCall intrinsicValue = 3).
	myCall premium: 4.
	self assert: (myCall timeValue = 1).
	
	"Put"
	myPut stockPrice: 48.
	myPut strikePrice: 50.
	self assert: (myPut intrinsicValue = 2).
	myPut premium: 3.
	self assert: (myPut timeValue = 1)! !

TestCase subclass: #StockPayoutRatioTest
	instanceVariableNames: 'myRatio'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockPayoutRatioTest commentStamp: 'rbc 8/16/2011 22:00' prior: 0!
Test the StockPayoutRatio class.!

!StockPayoutRatioTest methodsFor: 'setup' stamp: 'rbc 8/16/2011 21:38'!
setUp
	"Setup stock payout ratios test"
	myRatio := StockPayoutRatio new
! !

!StockPayoutRatioTest methodsFor: 'testing' stamp: 'rbc 8/16/2011 21:42'!
testDividendToCashFlow
	"Return dividend to cash flow. Pg 359."
	myRatio dividend: 1.
	myRatio cashFlow: 100.
	self assert: (myRatio dividendToCashFlow = (1/100))
	! !

!StockPayoutRatioTest methodsFor: 'testing' stamp: 'rbc 1/31/2012 18:30'!
testGets
	"Test setters and getters"

	"Cash flow"
	myRatio cashFlow: 100.
	self assert: (myRatio cashFlow = 100).
	
	myRatio net: 100.
	self assert: (myRatio net = 100).
	
	"Dividend paid on common"
	myRatio dividend: 1.
	self assert: (myRatio dividend = 1).
! !

!StockPayoutRatioTest methodsFor: 'testing' stamp: 'rbc 8/16/2011 21:46'!
testPayoutRatio
	"Test returning the payout ratio."
	myRatio dividend: 1.
	myRatio net: 100.
	self assert: (myRatio payoutRatio = (1/100))! !

TestCase subclass: #StockPerShareRatioTest
	instanceVariableNames: 'myRatio'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockPerShareRatioTest commentStamp: 'rbc 8/20/2011 09:28' prior: 0!
Test the StockPerShareRatio class.!

!StockPerShareRatioTest methodsFor: 'setup' stamp: 'rbc 8/18/2011 21:26'!
setUp
	"Setup a per share ratios test"
	myRatio := StockPerShareRatio new
! !

!StockPerShareRatioTest methodsFor: 'testing' stamp: 'rbc 8/18/2011 21:31'!
testBookValuePerShare
	"Test returning book value per share"
	myRatio bookValue: 120.
	myRatio goodWill: 10.
	myRatio intangible: 10.
	myRatio sharesBalanceSheetDate: 100.
	self assert: (myRatio bookValuePerShare = 1)
! !

!StockPerShareRatioTest methodsFor: 'testing' stamp: 'rbc 8/18/2011 21:40'!
testCashFlowPerShare
	"Test returning cash flow per share."
	myRatio cashFlow: 100.
	myRatio shares: 100.
	self assert: (myRatio cashFlowPerShare = 1)
! !

!StockPerShareRatioTest methodsFor: 'testing' stamp: 'rbc 8/18/2011 21:43'!
testCashPerShare
	"Test returning cash per share."
	myRatio cash: 110.
	myRatio claims: 10.
	myRatio sharesBalanceSheetDate: 100.
	self assert: (myRatio cashPerShare = 1)

	! !

!StockPerShareRatioTest methodsFor: 'testing' stamp: 'rbc 8/18/2011 21:50'!
testCurrentAssetsPerShare
	"Test returning current assets per share."
	myRatio assets: 102.
	myRatio claims: 2.
	myRatio sharesBalanceSheetDate: 100.
	self assert: (myRatio currentAssetsPerShare = 1)

	! !

!StockPerShareRatioTest methodsFor: 'testing' stamp: 'rbc 8/18/2011 21:53'!
testDividendPerShare
	"Test returning dividend per share."
	myRatio dividend: 1000.
	myRatio shares: 100.
	self assert: (myRatio dividendPerShare = 10)
! !

!StockPerShareRatioTest methodsFor: 'testing' stamp: 'rbc 8/18/2011 21:56'!
testEarningsPerShare
	"Test returning earnings per share."
	myRatio earnings: 1000.
	myRatio shares: 100.
	self assert: (myRatio earningsPerShare = 10)

	! !

!StockPerShareRatioTest methodsFor: 'testing' stamp: 'rbc 1/31/2012 19:35'!
testGets
	"Test setters and getters"
	myRatio bookValue: 120.
	self assert: (myRatio bookValue = 120).
	
	myRatio goodWill: 10.
	self assert: (myRatio goodWill = 10).

	myRatio intangible: 10.
	self assert: (myRatio intangible = 10).

	myRatio cashFlow: 100.
	self assert: (myRatio cashFlow = 100).

	myRatio shares: 100.
	self assert: (myRatio shares = 100).

	myRatio cash: 110.
	self assert: (myRatio cash = 110).

	myRatio claims: 10.
	self assert: (myRatio claims = 10).

	myRatio sharesBalanceSheetDate: 100.
	self assert: (myRatio sharesBalanceSheetDate = 100).

	myRatio assets: 102.
	self assert: (myRatio assets = 102).

	myRatio dividend: 1000.
	self assert: (myRatio dividend = 1000).

	myRatio earnings: 1000.
	self assert: (myRatio earnings = 1000).

	myRatio receivables: 5.
	self assert: (myRatio receivables = 5).

	myRatio sales: 1000.
	self assert: (myRatio sales = 1000)! !

!StockPerShareRatioTest methodsFor: 'testing' stamp: 'rbc 8/18/2011 22:01'!
testQuickAssetsPerShare
	"Test returning quick assets per share."
	myRatio cash: 105.
	myRatio receivables: 5.
	myRatio claims: 10.
	myRatio sharesBalanceSheetDate: 100.
	self assert: (myRatio quickAssetsPerShare = 1)

	! !

!StockPerShareRatioTest methodsFor: 'testing' stamp: 'rbc 8/18/2011 22:03'!
testSalesPerShare
	"Test returning sales per share"
	myRatio sales: 1000.
	myRatio shares: 100.
	self assert: (myRatio salesPerShare = 10)
! !

TestCase subclass: #StockPriceRatioTest
	instanceVariableNames: 'myRatio'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockPriceRatioTest commentStamp: 'rbc 8/20/2011 09:29' prior: 0!
Test the StockPriceRatio class.!

!StockPriceRatioTest methodsFor: 'setup' stamp: 'rbc 8/18/2011 22:07'!
setUp
	"Setup a price ratios test"
	myRatio := StockPriceRatio new
! !

!StockPriceRatioTest methodsFor: 'testing' stamp: 'rbc 8/18/2011 22:12'!
testDividendYield
	"Test returning the dividend yield"
	myRatio dividend: 0.01.
	myRatio price: 1.
	self assert: (myRatio dividendYield = 0.01)

	! !

!StockPriceRatioTest methodsFor: 'testing' stamp: 'rbc 8/18/2011 22:18'!
testEarningsYield
	"Test returning the earnings yield."
	myRatio eps: 0.03.
	myRatio price: 20.
	self assert: (myRatio earningsYield = 0.0015)! !

!StockPriceRatioTest methodsFor: 'testing' stamp: 'rbc 1/31/2012 19:15'!
testGets
	"Test setters and getters"

	myRatio dividend: 0.01.
	self assert: (myRatio dividend = 0.01).

	myRatio price: 1.
	self assert: (myRatio price = 1).

	myRatio eps: 0.03.
	self assert: (myRatio eps = 0.03).

	myRatio book: 5.
	self assert: (myRatio book = 5).

	myRatio sales: 5000.
	self assert: (myRatio sales = 5000).

	myRatio shares: 100.
	self assert: (myRatio shares = 100)! !

!StockPriceRatioTest methodsFor: 'testing' stamp: 'rbc 8/18/2011 22:21'!
testPriceEarningsRatio
	"Test returning the price-earnings ratio"
	myRatio price: 20.
	myRatio eps: 0.02.
	self assert: (myRatio priceEarningsRatio = 1000)! !

!StockPriceRatioTest methodsFor: 'testing' stamp: 'rbc 8/18/2011 22:25'!
testPriceToBook
	"Test returning the Price-to-book"
	myRatio price: 10.
	myRatio book: 5.
	self assert: (myRatio priceToBook = 2)! !

!StockPriceRatioTest methodsFor: 'testing' stamp: 'rbc 8/18/2011 22:31'!
testSalesPerDollarAtMarket
	"Test returning sales per dollar of common at market"
	myRatio sales: 5000.
	myRatio shares: 100.
	myRatio price: 5.
	self assert: (myRatio salesPerDollarAtMarket = 10)! !

TestCase subclass: #StockProfitabilityRatioTest
	instanceVariableNames: 'myRatio'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockProfitabilityRatioTest commentStamp: 'rbc 8/20/2011 09:29' prior: 0!
Test the StockProfitabilityRatio class.!

!StockProfitabilityRatioTest methodsFor: 'setup' stamp: 'rbc 8/20/2011 08:42'!
setUp
	"Setup a profitability ratios test"
	myRatio := StockProfitabilityRatio new
! !

!StockProfitabilityRatioTest methodsFor: 'testing' stamp: 'rbc 8/20/2011 08:46'!
testCapitalTurnover
	"Test returning capital turnover"
	myRatio sales: 100.
	myRatio tangibleAssets: 30.
	myRatio payables: 5.
	self assert: (myRatio capitalTurnover = 4)
	! !

!StockProfitabilityRatioTest methodsFor: 'testing' stamp: 'rbc 8/20/2011 08:57'!
testEarningsMargin
	"Test returning earnings margin"
	
	myRatio net: 190.
	myRatio minority: 5.
	myRatio taxAdjustedInterest: 5.
	myRatio sales: 1000.
	self assert: (myRatio earningsMargin = (2/10))
	! !

!StockProfitabilityRatioTest methodsFor: 'testing' stamp: 'rbc 1/31/2012 19:25'!
testGets
	"Test setters and getters"
	myRatio sales: 100.
	self assert: (myRatio sales = 100).

	myRatio tangibleAssets: 30.
	self assert: (myRatio tangibleAssets = 30).

	myRatio payables: 5.
	self assert: (myRatio payables = 5).

	myRatio net: 190.
	self assert: (myRatio net = 190).

	myRatio minority: 5.
	self assert: (myRatio minority = 5).

	myRatio taxAdjustedInterest: 5.
	self assert: (myRatio taxAdjustedInterest = 5).

	myRatio depreciation: 5.
	self assert: (myRatio depreciation = 5).

	myRatio preferredDividends: 5.
	self assert: (myRatio preferredDividends = 5).

	myRatio commonEquity: 1015.
	self assert: (myRatio commonEquity = 1015).

	myRatio goodwill: 5.
	self assert: (myRatio goodwill = 5).

	myRatio intangibles: 5.
	self assert: (myRatio intangibles = 5).

	myRatio deferredTaxLiability: 5.
	self assert: (myRatio deferredTaxLiability = 5)! !

!StockProfitabilityRatioTest methodsFor: 'testing' stamp: 'rbc 8/20/2011 09:04'!
testReturnOnCapital
	"Test returning, return on capital."
	myRatio net: 90.
	myRatio minority: 5.
	myRatio taxAdjustedInterest: 5.
	myRatio tangibleAssets: 55.
	myRatio payables: 5.
	self assert: (myRatio returnOnCapital = 2)
	! !

!StockProfitabilityRatioTest methodsFor: 'testing' stamp: 'rbc 8/20/2011 09:08'!
testReturnOnCapitalBeforeDepreciation
	"Test returning, return on capital before depreciation."
	myRatio net: 85.
	myRatio minority: 5.
	myRatio taxAdjustedInterest: 5.
	myRatio depreciation: 5.
	myRatio tangibleAssets: 15.
	myRatio payables: 5.
	self assert: (myRatio returnOnCapitalBeforeDepreciation = 10)! !

!StockProfitabilityRatioTest methodsFor: 'testing' stamp: 'rbc 8/20/2011 09:13'!
testReturnOnCommonEquity
	"Test returning, return on common equity."
	myRatio net: 105.
	myRatio preferredDividends: 5.
	myRatio commonEquity: 1015.
	myRatio goodwill: 5.
	myRatio intangibles: 5.
	myRatio deferredTaxLiability: 5.
	self assert: (myRatio returnOnCommonEquity = (1/10))
	! !

TestCase subclass: #StockStabilityRatioTest
	instanceVariableNames: 'myRatio'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockStabilityRatioTest commentStamp: 'rbc 8/20/2011 09:29' prior: 0!
Test the StockStabilityRatio class.!

!StockStabilityRatioTest methodsFor: 'setup' stamp: 'rbc 8/20/2011 09:17'!
setUp
	"Setup a stock stability ratios test"
	myRatio := StockStabilityRatio new
! !

!StockStabilityRatioTest methodsFor: 'testing' stamp: 'rbc 1/31/2012 19:01'!
testGets
	"Test setters and getters"

	myRatio rocWorstYear: 2.
	self assert: (myRatio rocWorstYear = 2).

	myRatio rocAvePast3Years: 4.
	self assert: (myRatio rocAvePast3Years = 4).

	myRatio seniorChargesAvePast3Years: 4.
	self assert: (myRatio seniorChargesAvePast3Years = 4).

	myRatio seniorChargesWorstYear: 2.
	self assert: (myRatio seniorChargesWorstYear = 2)! !

!StockStabilityRatioTest methodsFor: 'testing' stamp: 'rbc 8/20/2011 09:25'!
testMaxDeclineCoverSeniorCharges
	"Test returning the maximum decline in coverage of senior Charges."
	myRatio seniorChargesWorstYear: 2.
	myRatio seniorChargesAvePast3Years: 4.
	self assert: (myRatio maxDeclineCoverSeniorCharges = (1/2))
	! !

!StockStabilityRatioTest methodsFor: 'testing' stamp: 'rbc 8/20/2011 09:27'!
testPercentDeclineROC
	"Test returning percent decline in return on capital."
	myRatio rocWorstYear: 2.
	myRatio rocAvePast3Years: 4.
	self assert: (myRatio percentDeclineROC = (1/2))! !

TestCase subclass: #StockTest
	instanceVariableNames: 'myStock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockTest commentStamp: 'rbc 8/7/2011 12:52' prior: 0!
Testing for the stock class.
!

!StockTest methodsFor: 'setup' stamp: 'rbc 7/10/2011 13:24'!
setUp
	"Setup stock VSA test"
	myStock := Stock new.! !

!StockTest methodsFor: 'testing' stamp: 'rbc 7/10/2011 14:20'!
testActuality
	"Test setting the price"
	myStock stockPrice: 50.
	myStock numberShares: 100.
	self assert: (myStock actuality = 5000)! !

!StockTest methodsFor: 'testing' stamp: 'rbc 7/10/2011 13:24'!
testDescription
	"Test setting the stock description"
	| d |
	d := 'ABC Company'.
	myStock description: d.
	self assert: (myStock description = d)! !

!StockTest methodsFor: 'testing' stamp: 'rbc 7/10/2011 13:24'!
testNumberShares
	"Test setting the number of shares"
	myStock numberShares: 100.
	self assert: (myStock numberShares = 100)! !

!StockTest methodsFor: 'testing' stamp: 'rbc 7/10/2011 13:25'!
testPrice
	"Test setting the price"
	myStock stockPrice: 50.
	self assert: (myStock stockPrice = 50)! !

!StockTest methodsFor: 'testing' stamp: 'rbc 7/10/2011 13:25'!
testTicker
	"Test setting the ticker"
	| t |
	t := 'ABC'.
	myStock ticker: t.
	self assert: (myStock ticker = t)! !

TestCase subclass: #VSAAlertTest
	instanceVariableNames: 'alert'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAAlertTest commentStamp: 'rbc 7/28/2011 00:06' prior: 0!
Test alert message methods.
!

!VSAAlertTest methodsFor: 'setup' stamp: 'rbc 11/24/2011 10:13'!
setUp
alert := VSAAlert new.


! !

!VSAAlertTest methodsFor: 'testing' stamp: 'rbc 11/24/2011 10:15'!
testGet
| n |
alert actuality: 1.
alert capability: 1.
n := 'A VSA'.
alert vName: n.
self assert: (alert actuality = 1).
self assert: (alert capability = 1).
self assert: (alert vName = n)
! !

TestCase subclass: #VSABackupRestoreTest
	instanceVariableNames: 'myVSA'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSABackupRestoreTest commentStamp: 'rbc 1/27/2012 06:38' prior: 0!
Test VSABackupRestore.
!

!VSABackupRestoreTest methodsFor: 'setup' stamp: 'rbc 12/30/2011 23:27'!
setUp
	"Setup the test"
	myVSA := VSA new.
	! !

!VSABackupRestoreTest methodsFor: 'testing' stamp: 'rbc 1/27/2012 08:05'!
testSaveFetch
	"Test saving and fetching VSAs"
	| n a c p testVSA dir fileName |
	
	n := 'Test name'.
	a := 2.
	c := 4.
	p := 8.
	fileName := 'VSABackupRestoreTest'.
	dir := FileDirectory default.
	myVSA myExecutive vName: n.
	myVSA actuality: a.
	myVSA capability: c.
	myVSA potentiality: p.
	VSABackupRestore save: myVSA fileName: fileName.
	testVSA := VSABackupRestore fetch: fileName.
	self assert: (testVSA myExecutive vName = n).
	self assert: (testVSA actuality = a).
	self assert: (testVSA capability = c).
	self assert: (testVSA potentiality = p).
	dir deleteFileNamed: fileName! !

TestCase subclass: #VSAExecutiveMessageTest
	instanceVariableNames: 'message text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAExecutiveMessageTest commentStamp: 'rbc 9/8/2011 11:22' prior: 0!
Testing for policy message methods.
!

!VSAExecutiveMessageTest methodsFor: 'setup' stamp: 'rbc 9/8/2011 11:11'!
setUp
message := VSAPolicyMessage new
! !

!VSAExecutiveMessageTest methodsFor: 'testing' stamp: 'rbc 9/8/2011 11:13'!
testMessage
	"Test setting up a policy message"
	text := 'A test message'.
	message message: text.
	self assert: (message message = text)
! !

TestCase subclass: #VSAExecutiveTest
	instanceVariableNames: 'vExecutive'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAExecutiveTest commentStamp: 'rbc 7/28/2011 00:06' prior: 0!
Test executive setters and getters.
!

!VSAExecutiveTest methodsFor: 'setup' stamp: 'rbc 6/4/2011 18:52'!
setUp
vExecutive := VSAExecutive new
! !

!VSAExecutiveTest methodsFor: 'testing' stamp: 'rbc 6/4/2011 18:52'!
testGetName
	vExecutive vName: 'Test'.
	self assert: (vExecutive vName = 'Test')! !

!VSAExecutiveTest methodsFor: 'testing' stamp: 'rbc 6/4/2011 18:53'!
testInterval
	"Test setting and getting the communications interval"
	vExecutive interval: 30.
	self assert: (vExecutive interval = 30)! !

TestCase subclass: #VSAManagerModelTest
	instanceVariableNames: 'manager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAManagerModelTest commentStamp: 'rbc 3/1/2012 12:06' prior: 0!
A test of the model manager.
!

!VSAManagerModelTest methodsFor: 'setup' stamp: 'rbc 3/1/2012 11:59'!
setUp
	"Setup manager for testing"
	manager := VSAManagerModel new
! !

!VSAManagerModelTest methodsFor: 'testing' stamp: 'rbc 3/1/2012 12:06'!
testGets
	"Test accessors"
	| c t |

	"Put in a collection"
	c := OrderedCollection new.
	manager opsCollection: c.

	"Fetch and test it"
	t := manager opsCollection.
	self assert: (t isKindOf: OrderedCollection)! !

TestCase subclass: #VSAManagerReportTest
	instanceVariableNames: 'report'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAManagerReportTest commentStamp: 'rbc 7/28/2011 00:07' prior: 0!
Test manager report setters and getters.
!

!VSAManagerReportTest methodsFor: 'setup' stamp: 'rbc 9/8/2011 11:03'!
setUp
report := VSAManagerReport new.
report actuality: 1.
report capability: 1.
report latency: 1.
report performance: 1
! !

!VSAManagerReportTest methodsFor: 'testing' stamp: 'rbc 9/8/2011 11:04'!
testGet
self assert: (report actuality = 1).
self assert: (report capability = 1).
self assert: (report latency = 1).
self assert: (report performance = 1)! !

TestCase subclass: #VSAManagerTest
	instanceVariableNames: 'manager op op1 op2 msg'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAManagerTest commentStamp: 'rbc 7/28/2011 00:08' prior: 0!
This is a detailed test for manager methods.
!

!VSAManagerTest methodsFor: 'setup' stamp: 'rbc 6/4/2011 18:39'!
setUp
	"Setup manager for testing"
	manager := VSAManager new.

! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 7/18/2011 23:00'!
testCostCenter
	"Test profit center calculations"
	manager createOp: VSA.
	manager createOp: VSA.
	manager costCenter: true.
	op1 := manager getOp: 1.
	op2 := manager getOp: 2.
	op1 costCenter: true.
	op2 costCenter: true.
	op1 actuality: 50.
	op2 actuality: 50.
	op1 capability: 25.
	op2 capability: 25.
	op1 potentiality: 5.
	op2 potentiality: 5.
	self assert: (manager latency = (1/5)).
	self assert: (manager performance = (1/10)).
	self assert: (manager productivity = (1/2))
! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 1/28/2012 13:28'!
testEmptyCollection
	"Used to test method handling of an empty operations collection"
	self assert: (manager numberOfOperations = 0).

	"Testing"

	"Achievment"
	self assert: (manager latency = nil).
	self assert: (manager performance = nil).
	self assert: (manager productivity = nil).

	"Actuality"
	self assert: (manager actualitySum = nil).
	self assert: (manager actualityMin = nil).
	self assert: (manager actualityMean = nil).
	self assert: (manager actualityMax = nil).
	
	"Capability"
	self assert: (manager capabilitySum = nil).
	self assert: (manager capabilityMin = nil).
	self assert: (manager capabilityMean = nil).
	self assert: (manager capabilityMax = nil).
	
	"Potentiality"
	self assert: (manager potentialitySum = nil).
	self assert: (manager potentialityMin = nil).
	self assert: (manager potentialityMean = nil).
	self assert: (manager potentialityMax = nil)
! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 11/29/2011 00:00'!
testGettingAlerts
	"Test getting alerts"
	| c a |

	"Test retrieving an alert"
	manager createOp: VSA.
	op := manager getOp: 1.
	op actuality: 0.
	op capability: 2.
	op potentiality: 4.
	op alertVariance: 0.
	op mark: VSAAlert.
	c := manager getAlerts.
	a := c at: 1.
	self assert: (a isMemberOf: VSAAlert)! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 7/24/2011 09:05'!
testMovingOpsMessages
	"Test moving ops messages"
	manager createOp: VSA.
	manager createOp: VSA.
	op1 := manager getOp: 1.
	op2 := manager getOp: 2.
	msg := VSAOpsMessage new.
	msg to: 2.
	op1 mySupervisor outBox: msg.
	manager supervisorMoveOpsMessages.
	self assert: (op2 mySupervisor inBox isMemberOf: VSAOpsMessage)! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 6/4/2011 18:37'!
testOp
	"Test the creation and accessing of an operation"
	manager createOp: VSA.
	op := manager getOp: 1.
	self assert: (op isMemberOf: VSA).
	self assert: (manager capabilitySum = 1).
	self assert: (manager latency = 1).
	self assert: (manager numberOfOperations = 1).
	self assert: (manager performance = 1).
	self assert: (manager potentialitySum = 1).
	self assert: (manager productivity = 1)! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 1/28/2012 15:05'!
testOperationsDoNotCompare
	"Test nil throwing when operations do not compare"
	
	"Test setup"
	manager createOp: VSA.
	manager createOp: VSA.
	op1 := manager getOp: 1.
	op2 := manager getOp: 2.
	op1 actuality: 4.
	op2 actuality: 8.
	op1 capability: 10.
	op2 capability: 12.
	op1 potentiality: 14.
	op2 potentiality: 16.
	manager operationsCompare: false.

	"Testing"

	"operationsCompare getter"
	self assert: (manager operationsCompare = false).

	"Achievment"
	self assert: (manager latency = nil).
	self assert: (manager performance = nil).
	self assert: (manager productivity = nil).

	"Actuality"
	self assert: (manager actualitySum = nil).
	self assert: (manager actualityMin = nil).
	self assert: (manager actualityMean = nil).
	self assert: (manager actualityMax = nil).
	
	"Capability"
	self assert: (manager capabilitySum = nil).
	self assert: (manager capabilityMin = nil).
	self assert: (manager capabilityMean = nil).
	self assert: (manager capabilityMax = nil).
	
	"Potentiality"
	self assert: (manager potentialitySum = nil).
	self assert: (manager potentialityMin = nil).
	self assert: (manager potentialityMean = nil).
	self assert: (manager potentialityMax = nil)! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 7/18/2011 22:35'!
testProfitCenter
	"Test profit center calculations"
	manager createOp: VSA.
	manager createOp: VSA.
	op1 := manager getOp: 1.
	op2 := manager getOp: 2.
	op1 actuality: 20.
	op2 actuality: 20.
	op1 capability: 30.
	op2 capability: 30.
	op1 potentiality: 50.
	op2 potentiality: 50.
	self assert: (manager latency = (3/5)).
	self assert: (manager performance = (2/5)).
	self assert: (manager productivity = (2/3))
! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 6/4/2011 18:41'!
testReport
	"Test the creation and sending of a report"
	manager createOp: VSA.
	op := manager getOp: 1.
	manager sendManagerReport: VSAManagerReport.
	msg := manager outBox.
	self assert: (msg isMemberOf: VSAManagerReport)
! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 7/23/2011 23:49'!
testSendingOpsMessage
	"Test the creation and sending of an ops message"
	| m o |
	m := VSAOpsMessage new.
	m to: 1.
	manager createOp: VSA.
	o := manager getOp: 1.
	manager sendOpsMessage: m.
	self assert: (o mySupervisor inBox isMemberOf: VSAOpsMessage)
! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 1/28/2012 11:59'!
testStats
	"Test statistics"
	manager createOp: VSA.
	manager createOp: VSA.
	op1 := manager getOp: 1.
	op2 := manager getOp: 2.
	op1 actuality: 4.
	op2 actuality: 8.
	op1 capability: 10.
	op2 capability: 12.
	op1 potentiality: 14.
	op2 potentiality: 16.
	self assert: (manager actualityMin = 4).
	self assert: (manager actualityMean = 6).
	self assert: (manager actualityMax = 8).
	self assert: (manager capabilityMin = 10).
	self assert: (manager capabilityMean = 11).
	self assert: (manager capabilityMax = 12).
	self assert: (manager potentialityMin = 14).
	self assert: (manager potentialityMean = 15).
	self assert: (manager potentialityMax = 16)! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 2/29/2012 19:41'!
testsendingCopyOpsCollection
	"Test the creation and sending of a ops collection copy"
	| c v |
	
	"Create a VSA to go in the opsCollection"
	manager createOp: VSA.
	
	"Send an outbound copy of the ops collection"
	manager sendCopyOpsCollection: VSAOpsCopy.
	
	"Retrieve it from the out box to see what we have"
	msg := manager outBox.

	"Do we have a Ops copy message?"
	self assert: (msg isMemberOf: VSAOpsCopy).

	"Does it have a VSA in it?"
	
	"Get the collection out of the message"
	c := msg message.
	
	"Extract the Op/VSA that was created"
	v := c at: 1.
	
	"Test"
	self assert: (v isMemberOf: VSA)
! !

TestCase subclass: #VSAMessageTest
	instanceVariableNames: 'myMessage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAMessageTest commentStamp: 'rbc 11/25/2011 20:34' prior: 0!
Testing for basic VSA message methods.
!

!VSAMessageTest methodsFor: 'setup' stamp: 'rbc 11/24/2011 09:29'!
setUp
myMessage := VSAMessage new.! !

!VSAMessageTest methodsFor: 'testing' stamp: 'rbc 11/24/2011 10:17'!
testGets
| m n |
m := 'Some message text'.
n := 'A VSA name'.
myMessage setTimeStamp.
myMessage message: m.
myMessage vName: n.
self assert: (myMessage message = m).
self assert: (myMessage timeStamp isMemberOf: Array ).
self assert: (myMessage vName = n).
! !

TestCase subclass: #VSAModelTest
	instanceVariableNames: 'myVSA'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAModelTest commentStamp: 'rbc 3/1/2012 11:48' prior: 0!
Testing for the model VSA.
!

!VSAModelTest methodsFor: 'setup' stamp: 'rbc 3/1/2012 11:46'!
setUp
	"Setup test environment"
	myVSA := VSAModel new
! !

!VSAModelTest methodsFor: 'testing' stamp: 'rbc 3/1/2012 11:49'!
testInit
	"Just a stub to test initialization"
	self assert: (myVSA isMemberOf: VSAModel)! !

TestCase subclass: #VSAPlannerMessageTest
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAPlannerMessageTest commentStamp: 'rbc 9/8/2011 11:21' prior: 0!
Testing for planning message methods.!

!VSAPlannerMessageTest methodsFor: 'setup' stamp: 'rbc 9/8/2011 11:13'!
setUp
message := VSAPlannerMessage new
! !

!VSAPlannerMessageTest methodsFor: 'testing' stamp: 'rbc 9/8/2011 11:14'!
testGet
	"Test setting up a planner message"
	message potentiality: 1.
	self assert: (message potentiality = 1)! !

TestCase subclass: #VSAPlannerTest
	instanceVariableNames: 'myPlanner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAPlannerTest commentStamp: 'rbc 3/4/2012 12:08' prior: 0!
Testing for the facilities in the VSAPlanner.!

!VSAPlannerTest methodsFor: 'setup' stamp: 'rbc 3/4/2012 12:09'!
setUp
	"Setup the test environment"
	myPlanner := VSAPlanner new! !

!VSAPlannerTest methodsFor: 'testing' stamp: 'rbc 3/4/2012 12:57'!
testSortMail
	"Test sorting messages"
	| m o p t |

	"Create some messages"
	p := VSAPolicyMessage new.
	m := VSAManagerReport new.
	o := VSAOpsCopy new.

	"Put them in the inbox"
	myPlanner inBox: p.
	myPlanner inBox: m.
	myPlanner inBox: o.

	"Sort the mail"
	myPlanner sortMail.
	
	"Test to see that things were sorted"
	
	"Policy Message"
	t := myPlanner getPolicyMsg: 1.
	self assert: (t isMemberOf: VSAPolicyMessage).
	
	"Manager Report"
	t := myPlanner getManagerReport: 1.
	self assert: (t isMemberOf: VSAManagerReport).
	
	"Operations Copy"
	t := myPlanner opsCopy.
	self assert: (t isMemberOf: VSAOpsCopy)	
! !

TestCase subclass: #VSATest
	instanceVariableNames: 'myVSA'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSATest commentStamp: 'rbc 7/30/2011 10:26' prior: 0!
An end to end test for the VSA class, along with component classes.
!

!VSATest methodsFor: 'setup' stamp: 'rbc 6/3/2011 23:06'!
setUp
	"Setup test environment"
	myVSA := VSA new

! !

!VSATest methodsFor: 'testing' stamp: 'rbc 2/2/2012 18:58'!
testAlertsEnabled
	"Test operation of alertsEnabled accessors"
	myVSA alertsEnabled: false.
	self assert: (myVSA alertsEnabled = false)! !

!VSATest methodsFor: 'testing' stamp: 'rbc 7/18/2011 21:43'!
testCostCenter
	"Test cost center calculations"
	myVSA costCenter: true.
	myVSA actuality: 100.
	myVSA capability: 50.
	myVSA potentiality: 10.
	self assert: (myVSA latency = (1/5)).
	self assert: (myVSA performance = (1/10)).
	self assert: (myVSA productivity = (1/2))! !

!VSATest methodsFor: 'testing' stamp: 'rbc 12/30/2011 23:21'!
testGets
	"Test out get methods"
	myVSA actuality: 0.
	myVSA capability: 2.
	myVSA potentiality: 4.
	myVSA alertVariance: 0.
	myVSA mark: VSAAlert.
	self assert: (myVSA actuality = 0).
	self assert: (myVSA capability = 2).
	self assert: (myVSA potentiality = 4).
	self assert: (myVSA alertVariance = 0).
	self assert: (myVSA alert isMemberOf: VSAAlert).
	self assert: (myVSA myExecutive isMemberOf: VSAExecutive).
	self assert: (myVSA myPlanner isMemberOf: VSAPlanner).
	self assert: (myVSA myManager isMemberOf: VSAManager).
	self assert: (myVSA costCenter = false).
	self assert: (myVSA myManager costCenter = false).
	self assert: (myVSA operationsCompare = true).
	myVSA costCenter: true.
	self assert: (myVSA costCenter = true).
	self assert: (myVSA myManager costCenter = true)! !

!VSATest methodsFor: 'testing' stamp: 'rbc 2/29/2012 19:33'!
testMessaging
	"Test message sends"
	| msg pol t |
	myVSA myManager createOp: VSA.

	"Test the managers report"
	myVSA myManager sendManagerReport: VSAManagerReport.
	myVSA moveMessages.
	msg := myVSA myPlanner inBox.
	self assert: (msg isMemberOf: VSAManagerReport).
	msg := myVSA myExecutive inBox.
	self assert: (msg isMemberOf: VSAManagerReport).

	"Test sending a copy of the ops collection"
	myVSA myManager sendCopyOpsCollection: VSAOpsCopy.
	myVSA moveMessages.
	msg := myVSA myPlanner inBox.
	self assert: (msg isMemberOf: VSAOpsCopy).
	msg := myVSA myExecutive inBox.
	self assert: (msg isMemberOf: VSAOpsCopy).

	"Test sending a planner message"
	msg := VSAPlannerMessage new.
	myVSA myPlanner outBox: msg.
	myVSA moveMessages.
	t := myVSA myManager inBox.
	self assert: (t isMemberOf: VSAPlannerMessage).
	t := myVSA myExecutive inBox.
	self assert: (t isMemberOf: VSAPlannerMessage).

	"Test sending policies"
	pol := VSAPolicyMessage new.
	myVSA myExecutive outBox: pol.
	myVSA moveMessages.
	t := myVSA myPlanner inBox.
	self assert: (t isMemberOf: VSAPolicyMessage).
	t := myVSA myManager inBox.
	self assert: (t isMemberOf: VSAPolicyMessage)! !

!VSATest methodsFor: 'testing' stamp: 'rbc 1/18/2012 07:38'!
testProcesses
	"Test running processes"
	myVSA myExecutive interval: 1.
	myVSA startProcesses.
	myVSA runProcesses: false.
	self assert: (myVSA runProcesses = false)
	! !

!VSATest methodsFor: 'testing' stamp: 'rbc 7/18/2011 21:06'!
testProfitCenter
	"Test profit center calculations"
	myVSA actuality: 40.
	myVSA capability: 60.
	myVSA potentiality: 100.
	self assert: (myVSA latency = (3/5)).
	self assert: (myVSA performance = (2/5)).
	self assert: (myVSA productivity = (2/3)).
! !

!VSATest methodsFor: 'testing' stamp: 'rbc 9/24/2011 17:58'!
testUpdates
	"Test updates from subordinate operations"
	myVSA myManager createOp: VSA.
	myVSA update.
	self assert: (myVSA actuality = 1).
	self assert: (myVSA capability = 1).
	self assert: (myVSA potentiality = 1).
	self assert: (myVSA lastUpdate isMemberOf: Array )
! !

Object subclass: #GeospacialIndex
	instanceVariableNames: 'latitude longitude elevation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-CPS'!

!GeospacialIndex commentStamp: 'rbc 9/16/2011 12:59' prior: 0!
A class for defining the geospacial index.
!

!GeospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 21:09'!
elevation
	"Get the elevation"
	^elevation! !

!GeospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 21:09'!
elevation: aNumber
	"Set the elevation"
	elevation := aNumber! !

!GeospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 21:10'!
latitude
	"Get the latitude"
	^latitude! !

!GeospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 21:08'!
latitude: aNumber
	"Set the latitude"
	latitude := aNumber! !

!GeospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 21:10'!
longitude
	"Get the longitude"
	^longitude! !

!GeospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 21:08'!
longitude: aNumber
	"Set the longitude"
	longitude := aNumber! !

Object subclass: #HeatTransfer
	instanceVariableNames: 'transferRate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Heater'!

!HeatTransfer commentStamp: 'rbc 7/31/2011 16:46' prior: 0!
A simple class to remove heat from a room.
!

!HeatTransfer methodsFor: 'events-accessing' stamp: 'rbc 5/22/2011 11:49'!
transferHeat: anObject
	"Remove heat from an Object"
	| o n |
	o := anObject temperature.
	n := o - transferRate.
	anObject temperature: n.
	! !

!HeatTransfer methodsFor: 'accessing' stamp: 'rbc 5/21/2011 16:13'!
transferRate
	"Return the transfer rate"
	^transferRate.! !

!HeatTransfer methodsFor: 'accessing' stamp: 'rbc 5/21/2011 16:12'!
transferRate: aNumber
	"Set the transfer rate"
	transferRate := aNumber.! !

Object subclass: #Heater
	instanceVariableNames: 'desiredTemperature'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Heater'!

!Heater commentStamp: 'rbc 7/31/2011 16:46' prior: 0!
A room heater.
!

!Heater methodsFor: 'events-triggering' stamp: 'rbc 5/22/2011 11:44'!
HeatRoom: anObject
	"Heat a room object"
	anObject temperature: desiredTemperature.
	! !

!Heater methodsFor: 'accessing' stamp: 'rbc 5/22/2011 11:25'!
desiredTemperature
	"Set the desired temperature"
	^desiredTemperature.
	! !

!Heater methodsFor: 'accessing' stamp: 'rbc 5/22/2011 11:24'!
desiredTemperature: aNumber
	"Get the desired temperature"
	desiredTemperature := aNumber.
	! !

!Heater methodsFor: 'accessing' stamp: 'rbc 5/22/2011 11:31'!
setTemperature: aThermostatObject
	"Set the desired temperature"
	| t |
	t := aThermostatObject desiredTemperature.
	self desiredTemperature: t.	
	! !

Object subclass: #InfospacialIndex
	instanceVariableNames: 'globalNet subNet servicePoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-CPS'!

!InfospacialIndex commentStamp: 'rbc 9/12/2011 22:35' prior: 0!
A class for the infospacial index.
!

!InfospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 21:57'!
globalNet
	"Get the global network address"
	^globalNet! !

!InfospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 21:12'!
globalNet: aNumber
	"Set the global network address"
	globalNet := aNumber! !

!InfospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 21:57'!
servicePoint
	"Get the service point"
	^servicePoint! !

!InfospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 21:13'!
servicePoint: aNumber
	"Set the service point"
	servicePoint := aNumber! !

!InfospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 21:57'!
subNet
	"Get the subnet"
	^subNet! !

!InfospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 21:12'!
subNet: aNumber
	"Set the subnet"
	subNet := aNumber! !

Object subclass: #Room
	instanceVariableNames: 'temperature'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Heater'!

!Room commentStamp: 'rbc 7/31/2011 16:47' prior: 0!
A room.
!

!Room methodsFor: 'accessing' stamp: 'rbc 5/22/2011 10:47'!
temperature
	"Get the temperature"
	^temperature.
	! !

!Room methodsFor: 'accessing' stamp: 'rbc 5/22/2011 10:46'!
temperature: aNumber
	"Set the temperature"
	temperature := aNumber.
	! !

Object subclass: #SociospacialIndex
	instanceVariableNames: 'federation productionUnit commandActor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-CPS'!

!SociospacialIndex commentStamp: 'rbc 9/12/2011 22:36' prior: 0!
A class for the sociospacial index.
!

!SociospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 22:00'!
commandActor
	"Get the command actor"
	^commandActor! !

!SociospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 21:15'!
commandActor: aString
	"Set the command actor"
	commandActor := aString! !

!SociospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 22:00'!
federation
	"Get the federation"
	^federation! !

!SociospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 21:14'!
federation: aString
	"Set the federation"
	federation := aString! !

!SociospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 22:00'!
productionUnit
	"Get the production unit"
	^productionUnit! !

!SociospacialIndex methodsFor: 'accessing' stamp: 'rbc 9/12/2011 21:14'!
productionUnit: aString
	"Set the production unit"
	productionUnit := aString! !

Object subclass: #StockCreditRatio
	instanceVariableNames: 'currentAssets currentLiabilities inventories liabilities cashItems commonEquityBook tangibleAssets accruedPayables commonEquityMarket pretaxEarned seniorCharges cashFlowAfterTaxes taxAdjustedInterest rentals interest rent currentMaturities sinkingFundPayments cash receivables expenses depreciation nonCashCharges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockCreditRatio commentStamp: 'rbc 8/14/2011 16:01' prior: 0!
The purpose of this class is to calculate credit ratios.

Block, F., Cottle, S., Murray, R. (1988). Graham and Dodd's
Security Analysis (5th ed.). New York: McGraw-Hill Inc.
!

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 08:51'!
accruedPayables
	"Returns accrued payables"
	^ accruedPayables
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 08:52'!
accruedPayables: aNumber
	"Set accrued payables"
	accruedPayables := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 10:17'!
cash
	"Get cash"
	^ cash
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 10:17'!
cash: aNumber
	"Set cash"
	cash := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:18'!
cashFlowAfterTaxes
	"Get cash flow from operation after taxes"
	^ cashFlowAfterTaxes
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:17'!
cashFlowAfterTaxes: aNumber
	"Set cash flow from operation after taxes"
	cashFlowAfterTaxes := aNumber
	! !

!StockCreditRatio methodsFor: 'ratio' stamp: 'rbc 8/14/2011 11:45'!
cashFlowCoverSeniorCharges
	"Return cash flow coverage of senior charges.  Pg. 362."
	^ (cashFlowAfterTaxes + seniorCharges) / seniorCharges
	 ! !

!StockCreditRatio methodsFor: 'ratio' stamp: 'rbc 8/14/2011 13:40'!
cashFlowTotalCapital
	"Return cash flow to total capital. Pg 362."
	^ (cashFlowAfterTaxes + taxAdjustedInterest) / (tangibleAssets - accruedPayables)
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 06:06'!
cashItems
	"Return cash items."
	^ cashItems
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 06:04'!
cashItems: aNumber
	"Set the cash items"
	cashItems := aNumber
	! !

!StockCreditRatio methodsFor: 'ratio' stamp: 'rbc 8/14/2011 14:55'!
cashRatio
	"Return the cash ratio. Pg 359."
	^ cashItems / currentLiabilities
	
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 08:51'!
commonEquityBook
	"Return common equity at book value"
	^commonEquityBook
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 08:53'!
commonEquityBook: aNumber
	"Set the common equity at book value."
	commonEquityBook := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 06:47'!
commonEquityMarket
	"Returns common equity at market value."
	^ commonEquityMarket
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 06:44'!
commonEquityMarket: aNumber
	"Set the common equity at market value."
	commonEquityMarket := aNumber
	! !

!StockCreditRatio methodsFor: 'ratio' stamp: 'rbc 8/14/2011 15:05'!
coverageSeniorCharges
	"Return coverage of senior charges. Pg 361."
	^ pretaxEarned / seniorCharges
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 05:13'!
currentAssets
	"Return current assets"
	^ currentAssets
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 05:11'!
currentAssets: aNumber
	"Set current assets"
	currentAssets := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 05:16'!
currentLiabilities
	"Return current Liabilities"
	^ currentLiabilities
	
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 05:12'!
currentLiabilities: aNumber
	"Set current liabilities"
	currentLiabilities := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:51'!
currentMaturities
	"Get current maturities"
	^ currentMaturities
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:49'!
currentMaturities: aNumber
	"Set current maturities"
	currentMaturities := aNumber
	! !

!StockCreditRatio methodsFor: 'ratio' stamp: 'rbc 8/14/2011 15:07'!
currentRatio
	"Return the current ratio. Pg 359."
	^ currentAssets / currentLiabilities
	! !

!StockCreditRatio methodsFor: 'ratio' stamp: 'rbc 8/14/2011 15:19'!
defensiveIntervalDays
	"Return the defensive interval in days. Pg 363"
	^ ((cash + receivables) * 365) / (expenses - depreciation - nonCashCharges)
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 10:27'!
depreciation
	"Get depreciation"
	^ depreciation
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 10:26'!
depreciation: aNumber
	"Get depreciation"
	depreciation := aNumber
	! !

!StockCreditRatio methodsFor: 'ratio' stamp: 'rbc 8/14/2011 15:25'!
equityRatio
	"Returns equity ratio. Pg 359."
	^ commonEquityBook / (tangibleAssets - accruedPayables)
	! !

!StockCreditRatio methodsFor: 'ratio' stamp: 'rbc 8/14/2011 16:42'!
equityRatioMarket
	"Returns equity ratio at market. Pg 360."
	^ commonEquityMarket / (tangibleAssets - accruedPayables)
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 10:24'!
expenses
	"Get expenses"
	^ expenses
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 10:23'!
expenses: aNumber
	"Set expenses"
	expenses := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:44'!
interest
	"Get the interest"
	^ interest
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:42'!
interest: aNumber
	"Set interest"
	interest := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 05:36'!
inventories: aNumber
	"Set the inventory"
	inventories := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 05:37'!
liabilities: aNumber
	"Set liabilities"
	liabilities := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 10:38'!
nonCashCharges
	"Get other noncash charges"
	^ nonCashCharges
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 10:38'!
nonCashCharges: aNumber
	"Set other noncash charges"
	nonCashCharges := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:02'!
pretaxEarned
	"Get pretax earned for capital"
	^ pretaxEarned
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 8/14/2011 15:01'!
pretaxEarned: aNumber
	"Set pre-tax earned for capital"
	pretaxEarned := aNumber
	! !

!StockCreditRatio methodsFor: 'ratio' stamp: 'rbc 8/14/2011 16:43'!
quickRatio
	"Return the quick ratio. Pg 359."
	^ (currentAssets - inventories) / liabilities
! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 10:21'!
receivables
	"Get receivables"
	^ receivables
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 10:20'!
receivables: aNumber
	"Set receivables"
	receivables := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:47'!
rent
	"Get the rent"
	^ rent
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:46'!
rent: aNumber
	"Set the rent"
	rent := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:40'!
rentals
	"Get the rentals"
	^ rentals
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:39'!
rentals: aNumber
	"Set the rentals"
	rentals := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:08'!
seniorCharges
	"Return the senior charges"
	^ seniorCharges
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:05'!
seniorCharges: aNumber
	"Set senior charges"
	seniorCharges := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:55'!
sinkingFundPayments
	"Get Sinking Fund Payments"
	^ sinkingFundPayments
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:54'!
sinkingFundPayments: aNumber
	"Set sinking fund payments"
	sinkingFundPayments := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 06:26'!
tangibleAssets
	"Return Tangible Assets."
	^ tangibleAssets
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 08:53'!
tangibleAssets: aNumber
	"Set tangible assets"
	tangibleAssets := aNumber
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:29'!
taxAdjustedInterest
	"Get tax-adjusted interest"
	^ taxAdjustedInterest
	! !

!StockCreditRatio methodsFor: 'accessing' stamp: 'rbc 10/26/2007 09:27'!
taxAdjustedInterest: aNumber
	"Set tax-adjusted interest"
	taxAdjustedInterest := aNumber
	! !

!StockCreditRatio methodsFor: 'ratio' stamp: 'rbc 8/14/2011 15:55'!
totalDebtServiceCoverage
	"Return total debt service coverage. Pg 362"
	^ (cashFlowAfterTaxes + rentals + taxAdjustedInterest) / (interest + rent + currentMaturities + sinkingFundPayments)
	
	! !

Object subclass: #StockGrowthRatio
	instanceVariableNames: 'finalSales baseSales netEarnedFinal netEarnedBase epsInFinalPeriod epsInBasePeriod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockGrowthRatio commentStamp: 'rbc 8/7/2011 12:50' prior: 0!
The purpose of the StockGrowthRatio class is to calculate growth ratios.

Block, F., Cottle, S., Murray, R. (1988). Graham and Dodd's
Security Analysis (5th ed.). New York: McGraw-Hill Inc.
!

!StockGrowthRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 19:17'!
baseSales
	"Get sales in the base period"
	^ baseSales
	! !

!StockGrowthRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 19:17'!
baseSales: aNumber
	"Set sales in the base period"
	baseSales := aNumber
	! !

!StockGrowthRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 20:02'!
epsInBasePeriod
	"Get earnings per share in the base period"
	^ epsInBasePeriod
	! !

!StockGrowthRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 20:01'!
epsInBasePeriod: aNumber
	"Set earnings per share in the base period"
	epsInBasePeriod := aNumber
	! !

!StockGrowthRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 19:55'!
epsInFinalPeriod
	"Get earnings per share in the final period"
	^ epsInFinalPeriod
	! !

!StockGrowthRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 19:54'!
epsInFinalPeriod: aNumber
	"Set earnings per share in the final period"
	epsInFinalPeriod := aNumber
	! !

!StockGrowthRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 19:07'!
finalSales
	"Get sales in the final period"
	^ finalSales
	! !

!StockGrowthRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 19:07'!
finalSales: aNumber
	"Set sales in the final period"
	finalSales := aNumber
	! !

!StockGrowthRatio methodsFor: 'ratio' stamp: 'rbc 8/16/2011 20:48'!
growthInEPS
	"Return growth in earnings per share. Pg 355."
	^ epsInFinalPeriod / epsInBasePeriod
	! !

!StockGrowthRatio methodsFor: 'ratio' stamp: 'rbc 8/16/2011 21:05'!
growthInSales
	"Return growth in sales. Pg 355."
	^ finalSales / baseSales
	! !

!StockGrowthRatio methodsFor: 'ratio' stamp: 'rbc 8/16/2011 21:19'!
growthInTotalReturn
	"Get growth in total return. Pg 355"
	^ netEarnedFinal / netEarnedBase
	! !

!StockGrowthRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 19:30'!
netEarnedBase
	"Get net earned for total capital in base period"
	^ netEarnedBase
	! !

!StockGrowthRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 19:29'!
netEarnedBase: aNumber
	"Set net earned for total capital in base period"
	netEarnedBase := aNumber
	! !

!StockGrowthRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 19:23'!
netEarnedFinal
	"Get net earned for total capital in the final period"
	^ netEarnedFinal
	! !

!StockGrowthRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 19:24'!
netEarnedFinal: aNumber
	"Set net earned for total capital in the final period"
	netEarnedFinal := aNumber
	! !

Object subclass: #StockMiscRatio
	instanceVariableNames: 'depreciation sales grossPlant costOfGoodsSold inventory receivables'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockMiscRatio commentStamp: 'rbc 8/7/2011 12:50' prior: 0!
The purpose of the StockMiscRatio class is to provide ratios not found in other classes.

Block, F., Cottle, S., Murray, R. (1988). Graham and Dodd's
Security Analysis (5th ed.). New York: McGraw-Hill Inc.
!

!StockMiscRatio methodsFor: 'ratio' stamp: 'rbc 8/16/2011 21:18'!
accountsReceivableTurnover
	"Return accounts receivable turnover. Pg 363."
	^ sales / receivables
	! !

!StockMiscRatio methodsFor: 'accessing' stamp: 'rbc 8/16/2011 21:30'!
costOfGoodsSold
	"Get the cost of goods sold."
	^costOfGoodsSold! !

!StockMiscRatio methodsFor: 'accessing' stamp: 'rbc 8/16/2011 21:30'!
costOfGoodsSold: aNumber
	"Set the cost of goods sold."
	costOfGoodsSold := aNumber! !

!StockMiscRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 23:53'!
depreciation
	"Get depreciation expense"
	^ depreciation
	! !

!StockMiscRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 23:52'!
depreciation: aNumber
	"Set depreciation expense"
	depreciation := aNumber
	! !

!StockMiscRatio methodsFor: 'ratio' stamp: 'rbc 8/16/2011 21:18'!
depreciationToGrossPlant
	"Returns depreciation to gross plant. Pg 363."
	^ depreciation / grossPlant
	! !

!StockMiscRatio methodsFor: 'ratio' stamp: 'rbc 8/16/2011 21:19'!
depreciationToSales
	"Return depreciation to sales. Pg 363."
	^ depreciation / sales
	! !

!StockMiscRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 23:54'!
grossPlant
	"Get gross plant"
	^ grossPlant
	! !

!StockMiscRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 23:54'!
grossPlant: aNumber
	"Set gross plant"
	grossPlant := aNumber
	! !

!StockMiscRatio methodsFor: 'accessing' stamp: 'rbc 10/28/2007 00:07'!
inventory
	"Get inventory including lifo researve (if any)"
	^ inventory.
	! !

!StockMiscRatio methodsFor: 'accessing' stamp: 'rbc 10/28/2007 00:06'!
inventory: aNumber
	"Set inventory including lifo researve (if any)"
	inventory := aNumber
	! !

!StockMiscRatio methodsFor: 'ratio' stamp: 'rbc 8/16/2011 21:31'!
inventoryTurnover
	"Returns inventory turnover. Pg 363."
	^ costOfGoodsSold / inventory
	! !

!StockMiscRatio methodsFor: 'accessing' stamp: 'rbc 10/28/2007 00:09'!
receivables
	"Get accounts receivable"
	^ receivables
	! !

!StockMiscRatio methodsFor: 'accessing' stamp: 'rbc 10/28/2007 00:09'!
receivables: aNumber
	"Set accounts receivable"
	receivables := aNumber
	! !

!StockMiscRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 23:41'!
sales
	"Get sales"
	^ sales
	! !

!StockMiscRatio methodsFor: 'accessing' stamp: 'rbc 10/27/2007 23:40'!
sales: aNumber
	"Set sales"
	sales := aNumber
	! !

Object subclass: #StockOptionQuote
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockOptionQuote commentStamp: 'rbc 8/20/2011 09:56' prior: 0!
A stub for stock option quote aquisition.!

Object subclass: #StockPayoutRatio
	instanceVariableNames: 'dividend net cashFlow'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockPayoutRatio commentStamp: 'rbc 8/7/2011 12:50' prior: 0!
The StockPayoutRatios class is used to calculate payout ratios.

Block, F., Cottle, S., Murray, R. (1988). Graham and Dodd's
Security Analysis (5th ed.). New York: McGraw-Hill Inc.
!

!StockPayoutRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 07:20'!
cashFlow
	"Get cash flow from operations after taxes"
	^ cashFlow
	! !

!StockPayoutRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 07:20'!
cashFlow: aNumber
	"Set cash flow from operations after taxes"
	cashFlow := aNumber
	! !

!StockPayoutRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 07:18'!
dividend
	"Get dividend paid on common"
	^ dividend
	! !

!StockPayoutRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 07:17'!
dividend: aNumber
	"Set dividend paid on common"
	dividend := aNumber
	! !

!StockPayoutRatio methodsFor: 'ratio' stamp: 'rbc 8/16/2011 21:40'!
dividendToCashFlow
	"Return dividend to cash flow. Pg 359."
	^ dividend / cashFlow
	! !

!StockPayoutRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 07:19'!
net
	"Get net income available to common"
	^ net
	! !

!StockPayoutRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 07:19'!
net: aNumber
	"Set net income available to common"
	net := aNumber
	! !

!StockPayoutRatio methodsFor: 'ratio' stamp: 'rbc 8/16/2011 21:44'!
payoutRatio
	"Return payout ratio. Pg 358."
	^ dividend / net
	! !

Object subclass: #StockPerShareRatio
	instanceVariableNames: 'earnings shares dividend sales cashFlow bookValue goodWill intangible sharesBalanceSheetDate assets claims cash receivables'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockPerShareRatio commentStamp: 'rbc 8/7/2011 12:51' prior: 0!
The purpose of the StockPerShareRatios class is to provide per share calculations.

Block, F., Cottle, S., Murray, R. (1988). Graham and Dodd's
Security Analysis (5th ed.). New York: McGraw-Hill Inc.
!

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 14:32'!
assets
	"Get current assets"
	^ assets
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 14:31'!
assets: aNumber
	"Set current assets"
	assets := aNumber
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 14:00'!
bookValue
	"Get book value of common equity"
	^ bookValue
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 13:51'!
bookValue: aNumber
	"Set book value of common equity"
	bookValue := aNumber
	! !

!StockPerShareRatio methodsFor: 'ratio' stamp: 'rbc 8/18/2011 21:36'!
bookValuePerShare
	"Return book value per share. Pg 335."
	^ (bookValue - goodWill - intangible) / sharesBalanceSheetDate
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 14:50'!
cash
	"Get cash"
	^ cash
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 14:50'!
cash: aNumber
	"Set cash"
	cash := aNumber
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 12:52'!
cashFlow
	"Get cash flow from operations after taxes"
	^ cashFlow
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 12:51'!
cashFlow: aNumber
	"Set cash flow from operations after taxes"
	cashFlow := aNumber
	! !

!StockPerShareRatio methodsFor: 'ratio' stamp: 'rbc 8/18/2011 21:36'!
cashFlowPerShare
	"Return cash flow per share. Pg 335."
	^ cashFlow / shares! !

!StockPerShareRatio methodsFor: 'ratio' stamp: 'rbc 8/18/2011 21:41'!
cashPerShare
	"Return cash per share.  Pg 335."
	^ (cash - claims) / sharesBalanceSheetDate
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 14:37'!
claims
	"Get all claims prior to common"
	^ claims
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 14:37'!
claims: aNumber
	"Set all claims prior to common"
	claims := aNumber
	! !

!StockPerShareRatio methodsFor: 'ratio' stamp: 'rbc 8/18/2011 21:48'!
currentAssetsPerShare
	"Return current assets per share. Pg 355."
	^ (assets - claims) / sharesBalanceSheetDate
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 12:41'!
dividend
	"Get total annual dividends paid to common"
	^ dividend
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 12:40'!
dividend: aNumber
	"Set total annual dividends paid to common"
	dividend := aNumber
	! !

!StockPerShareRatio methodsFor: 'ratio' stamp: 'rbc 8/18/2011 21:51'!
dividendPerShare
	"Return dividend per share. Pg 355."
	^ dividend / shares
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 11:56'!
earnings
	"Get the earnings available for the common shares"
	^ earnings
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 11:56'!
earnings: aNumber
	"Set the earnings available for the common shares"
	earnings := aNumber
	! !

!StockPerShareRatio methodsFor: 'ratio' stamp: 'rbc 8/18/2011 21:54'!
earningsPerShare
	"Return earnings per share. Pg 355."
	^ earnings / shares
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 14:16'!
goodWill
	"Get good will"
	^ goodWill
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 14:13'!
goodWill: aNumber
	"Set good will"
	goodWill := aNumber
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 14:16'!
intangible
	"Get most other intangible assets"
	^ intangible
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 14:15'!
intangible: aNumber
	"Set most other intangible assets"
	intangible := aNumber
	! !

!StockPerShareRatio methodsFor: 'ratio' stamp: 'rbc 8/18/2011 21:57'!
quickAssetsPerShare
	"Return quick assets per share. Pg 335."
	^ (cash + receivables - claims) / sharesBalanceSheetDate
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 14:52'!
receivables
	"Get receivables"
	^ receivables
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 14:51'!
receivables: aNumber
	"Set receivables"
	receivables := aNumber
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 12:46'!
sales
	"Get total sales"
	^ sales
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 12:46'!
sales: aNumber
	"Set total sales"
	sales := aNumber
	! !

!StockPerShareRatio methodsFor: 'ratio' stamp: 'rbc 8/18/2011 22:04'!
salesPerShare
	"Return sales per share. Pg 335."
	^ sales / shares
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 12:01'!
shares
	"Get the weighted average common shares outstanding"
	^ shares
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 12:01'!
shares: aNumber
	"Set the weighted average common shares outstanding"
	shares := aNumber
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 14:26'!
sharesBalanceSheetDate
	"Get common shares outstanding at balance sheet date"
	^ sharesBalanceSheetDate
	! !

!StockPerShareRatio methodsFor: 'accessing' stamp: 'rbc 10/29/2007 14:26'!
sharesBalanceSheetDate: aNumber
	"Set common shares outstanding at balance sheet date"
	sharesBalanceSheetDate := aNumber
	! !

Object subclass: #StockPriceRatio
	instanceVariableNames: 'price eps dividend sales shares book'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockPriceRatio commentStamp: 'rbc 10/30/2007 11:21' prior: 0!
A class to calculate price ratios

Block, F., Cottle, S., Murray, R. (1988). Graham and Dodd's
Security Analysis (5th ed.). New York: McGraw-Hill Inc.!

!StockPriceRatio methodsFor: 'accessing' stamp: 'rbc 10/30/2007 11:32'!
book
	"Get book value per share"
	^ book
	! !

!StockPriceRatio methodsFor: 'accessing' stamp: 'rbc 10/30/2007 11:32'!
book: aNumber
	"Set book value per share"
	book := aNumber
	! !

!StockPriceRatio methodsFor: 'accessing' stamp: 'rbc 10/30/2007 11:17'!
dividend
	"Get dividend per share"
	^ dividend
	! !

!StockPriceRatio methodsFor: 'accessing' stamp: 'rbc 10/30/2007 11:16'!
dividend: aNumber
	"Set dividend per share"
	dividend := aNumber
	! !

!StockPriceRatio methodsFor: 'ratio' stamp: 'rbc 8/18/2011 22:14'!
dividendYield
	"Return the dividend yield. Pg 345."
	^ dividend / price
	! !

!StockPriceRatio methodsFor: 'ratio' stamp: 'rbc 8/18/2011 22:15'!
earningsYield
	"Return the earnings yield. Pg 345."
	^ eps / price
	! !

!StockPriceRatio methodsFor: 'accessing' stamp: 'rbc 10/30/2007 06:00'!
eps
	"Get earnings per share"
	^ eps
	! !

!StockPriceRatio methodsFor: 'accessing' stamp: 'rbc 10/30/2007 06:00'!
eps: aNumber
	"Set earnings per share"
	eps := aNumber
	! !

!StockPriceRatio methodsFor: 'accessing' stamp: 'rbc 10/30/2007 05:59'!
price
	"Get price per share"
	^ price
	! !

!StockPriceRatio methodsFor: 'accessing' stamp: 'rbc 10/30/2007 05:56'!
price: aNumber
	"Set the price per share"
	price := aNumber
	! !

!StockPriceRatio methodsFor: 'ratio' stamp: 'rbc 8/18/2011 22:26'!
priceEarningsRatio
	"Return the price-earnings ratio. Pg 345."
	^ price / eps
	! !

!StockPriceRatio methodsFor: 'ratio' stamp: 'rbc 8/18/2011 22:24'!
priceToBook
	"Return the Price-to-book. Pg 345."
	^ price / book
	! !

!StockPriceRatio methodsFor: 'accessing' stamp: 'rbc 10/30/2007 11:23'!
sales
	"Get sales"
	^ sales
	! !

!StockPriceRatio methodsFor: 'accessing' stamp: 'rbc 10/30/2007 11:23'!
sales: aNumber
	"Set sales"
	sales := aNumber
	! !

!StockPriceRatio methodsFor: 'ratio' stamp: 'rbc 8/18/2011 22:26'!
salesPerDollarAtMarket
	"Return sales per dollar of common at market. Pg 345."
	^ sales / (shares * price)
	! !

!StockPriceRatio methodsFor: 'accessing' stamp: 'rbc 10/30/2007 11:30'!
shares
	"Get weighted average shares outstanding"
	^ shares
	! !

!StockPriceRatio methodsFor: 'accessing' stamp: 'rbc 10/30/2007 11:24'!
shares: aNumber
	"Set weighted average shares outstanding"
	shares := aNumber
	! !

Object subclass: #StockProfitabilityRatio
	instanceVariableNames: 'net minority taxAdjustedInterest tangibleAssets payables sales depreciation preferredDividends commonEquity goodwill intangibles deferredTaxLiability'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockProfitabilityRatio commentStamp: 'rbc 8/7/2011 12:51' prior: 0!
The purpose of the StockProfitabilityRatio class is to calculate common financial profitability ratios.

Block, F., Cottle, S., Murray, R. (1988). Graham and Dodd's
Security Analysis (5th ed.). New York: McGraw-Hill Inc.
!

!StockProfitabilityRatio methodsFor: 'ratio' stamp: 'rbc 8/20/2011 08:50'!
capitalTurnover
	"Return capital turnover. Pg 352."
	^ sales / (tangibleAssets - payables)
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 14:17'!
commonEquity
	"Get common equity"
	^ commonEquity
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 14:17'!
commonEquity: aNumber
	"Set common equity"
	commonEquity := aNumber
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 14:30'!
deferredTaxLiability
	"Get deferred tax liability"
	^ deferredTaxLiability
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 14:30'!
deferredTaxLiability: aNumber
	"Set deferred tax liability"
	deferredTaxLiability := aNumber
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 13:59'!
depreciation
	"Get depreciation"
	^ depreciation
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 13:59'!
depreciation: aNumber
	"Set depreciation"
	depreciation := aNumber
	! !

!StockProfitabilityRatio methodsFor: 'ratio' stamp: 'rbc 8/20/2011 08:50'!
earningsMargin
	"Return earnings margin. Pg 352."
	^ (net + minority + taxAdjustedInterest) / sales
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 14:22'!
goodwill
	"Get goodwill"
	^ goodwill
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 14:21'!
goodwill: aNumber
	"Set goodwill"
	goodwill := aNumber
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 14:28'!
intangibles
	"Get most intangible assets"
	^ intangibles
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 14:28'!
intangibles: aNumber
	"Set most intangible assets"
	intangibles := aNumber
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 13:17'!
minority
	"Get minority interest"
	^ minority
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 13:16'!
minority: aNumber
	"Set minority interest"
	minority := aNumber.
	
! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 13:15'!
net
	"Get net income"
	^ net
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 13:14'!
net: aNumber
	"Set net income"
	net := aNumber
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 13:46'!
payables
	"Get short-term accrued payables"
	^ payables
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 13:45'!
payables: aNumber
	"Set short-term accrued payables"
	payables := aNumber
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 14:14'!
preferredDividends
	"Get preferred dividends"
	^ preferredDividends
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 14:14'!
preferredDividends: aNumber
	"Set preferred dividends"
	preferredDividends := aNumber
	! !

!StockProfitabilityRatio methodsFor: 'ratio' stamp: 'rbc 8/20/2011 08:51'!
returnOnCapital
	"Return return on capital. Pg 352."
	^ (net + minority + taxAdjustedInterest) / (tangibleAssets - payables)
	! !

!StockProfitabilityRatio methodsFor: 'ratio' stamp: 'rbc 8/20/2011 08:51'!
returnOnCapitalBeforeDepreciation
	"Return return on capital before depreciation. Pg 353."
	^ (net + minority + taxAdjustedInterest + depreciation)
	    / (tangibleAssets - payables)! !

!StockProfitabilityRatio methodsFor: 'ratio' stamp: 'rbc 8/20/2011 08:52'!
returnOnCommonEquity
	"Return return on common equity. Pg 354."
	^ (net - preferredDividends)
	    / (commonEquity - goodwill - intangibles - deferredTaxLiability)
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 13:52'!
sales
	"Get sales"
	^ sales
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 13:51'!
sales: aNumber
	"Set sales"
	sales := aNumber
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 13:42'!
tangibleAssets
	"Get tangible assets"
	^ tangibleAssets
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 13:42'!
tangibleAssets: aNumber
	"Set tangible assets"
	tangibleAssets := aNumber
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 13:38'!
taxAdjustedInterest
	"Get tax-adjusted interest"
	^ taxAdjustedInterest
	! !

!StockProfitabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 13:37'!
taxAdjustedInterest: aNumber
	"Set tax-adjusted interest"
	taxAdjustedInterest := aNumber
	! !

Object subclass: #StockQuote
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockQuote commentStamp: 'rbc 8/7/2011 12:51' prior: 0!
A class for getting stock quotes.
!

Object subclass: #StockStabilityRatio
	instanceVariableNames: 'seniorChargesWorstYear seniorChargesAvePast3Years rocWorstYear rocAvePast3Years'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockStabilityRatio commentStamp: 'rbc 8/20/2011 09:56' prior: 0!
Stability measurements for non-growth companies.

Block, F., Cottle, S., Murray, R. (1988). Graham and Dodd's
Security Analysis (5th ed.). New York: McGraw-Hill Inc.!

!StockStabilityRatio methodsFor: 'ratio' stamp: 'rbc 8/20/2011 09:18'!
maxDeclineCoverSeniorCharges
	"Return the maximum decline in coverage of senior Charges. Pg 357."
	^ seniorChargesWorstYear / seniorChargesAvePast3Years
	! !

!StockStabilityRatio methodsFor: 'ratio' stamp: 'rbc 8/20/2011 09:18'!
percentDeclineROC
	"Return percent decline in return on capital. Pg 357."
	^ rocWorstYear / rocAvePast3Years
	! !

!StockStabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 15:15'!
rocAvePast3Years
	"Get the average of the past three years"
	^ rocAvePast3Years
	! !

!StockStabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 15:15'!
rocAvePast3Years: aNumber
	"Set the average of the past three years"
	rocAvePast3Years := aNumber
	! !

!StockStabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 15:12'!
rocWorstYear
	"Get the worst year"
	^ rocWorstYear
	! !

!StockStabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 15:12'!
rocWorstYear: aNumber
	"Set the worst year"
	rocWorstYear := aNumber
	! !

!StockStabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 15:08'!
seniorChargesAvePast3Years
	"Get average of previous three years"
	^ seniorChargesAvePast3Years
	! !

!StockStabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 15:08'!
seniorChargesAvePast3Years: aNumber
	"Set average of previous three years"
	seniorChargesAvePast3Years := aNumber
	! !

!StockStabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 15:04'!
seniorChargesWorstYear
	"Get worst year"
	^ seniorChargesWorstYear
	! !

!StockStabilityRatio methodsFor: 'accessing' stamp: 'rbc 11/3/2007 15:05'!
seniorChargesWorstYear: aNumber
	"Set worst year"
	seniorChargesWorstYear := aNumber
	! !

Object subclass: #Thermostat
	instanceVariableNames: 'desiredTemperature'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Heater'!

!Thermostat commentStamp: 'rbc 7/31/2011 16:47' prior: 0!
A thermostat.
!

!Thermostat methodsFor: 'accessing' stamp: 'rbc 5/22/2011 10:49'!
desiredTemperature
	"Get the desired temperature"
	^desiredTemperature.
	! !

!Thermostat methodsFor: 'accessing' stamp: 'rbc 5/22/2011 10:48'!
desiredTemperature: aNumber
	"Set the desired temperature"
	desiredTemperature := aNumber.
	! !

Object subclass: #VSA
	instanceVariableNames: 'mySupervisor myExecutive myPlanner myManager actuality capability potentiality alerts alertsEnabled alertVariance runProcesses costCenter operationsCompare lastUpdate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSA commentStamp: 'rbc 1/28/2012 17:42' prior: 0!
Both a viable system and operation (system 1).  Each VSA starts an Executive (5),
Planner (4) and Manager (3).  A VSA is also paired with a Supervisor (2) in its role as
an operation.

The Viable System Agent, or VSA is a software implementation of
the Viable System Model (VSM), a model for autonomous systems
developed by Stafford Beer. The VSA provides structure,
communications, auditing and alerting as defined in the VSM. The
VSA is used to represent both viable systems and subordinate
viable systems. This is called recursion in the VSM. The VSA is
intended to facilitate creating and managing autonomous software
systems.

Viable System Terminology:

1) Operation(s) - A primary system activity
2) Supervisor(s) - Coordinates operations
3) Manager - Plans and controls current operations
4) Planner - Monitors environment and plans for the future
5) Executive - Maintains system identity

Operation Description:

-A primary activity that performs part of the transformation of the viable system
-Generates algedonic alerts that are sent to Manager
-Responds to audit messages from Manager
-Communicates with the paired Supervisor

References:

-1972, Stafford Beer, Brain of the Firm; Allen Lane, The Penguin Press, London, Herder and Herder, USA.
-1979, Stafford Beer, The Heart of Enterprise; John Wiley, London and New York.

Copyright 2011 Robert Bruce Carleton. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY ROBERT BRUCE CARLETON ''AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ROBERT BRUCE CARLETON OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those of the
authors and should not be interpreted as representing official policies, either expressed
or implied, of Robert Bruce Carleton.
!

VSA subclass: #CyberPhysicalSystem
	instanceVariableNames: 'myGSI mySSI'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-CPS'!

!CyberPhysicalSystem commentStamp: 'rbc 9/12/2011 22:37' prior: 0!
The cyber physical system.

References:

-Bayne, Jay S. Cyberspatial Mechanics, IEEE Transactions on Systems, Man, and Cybernetics-part b: Cybernetics, Vol. 38, No. 3, June 2008. IEEE.
-Beer, Stafford. The Heart of Enterprise (1979). Chichester: John Wiley & Sons.
-Beer, Stafford. Brain of the Firm (2nd ed./1981). Chichester: John Wiley & Sons.

Copyright 2011 Robert Bruce Carleton. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY ROBERT BRUCE CARLETON ''AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ROBERT BRUCE CARLETON OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those of the
authors and should not be interpreted as representing official policies, either expressed
or implied, of Robert Bruce Carleton.!

!CyberPhysicalSystem methodsFor: 'initialization' stamp: 'rbc 2/3/2012 17:18'!
createVSA
	"Pick up VSA defaults"
	super createVSA.
	"Add the CPS address objects"
	self myGSI: GeospacialIndex.
	self mySSI: SociospacialIndex! !

!CyberPhysicalSystem methodsFor: 'accessing' stamp: 'rbc 9/12/2011 18:05'!
myGSI
	"Get the geo spacial index"
	^myGSI
	! !

!CyberPhysicalSystem methodsFor: 'accessing' stamp: 'rbc 9/12/2011 18:05'!
myGSI: anObject
	"Create the geo spacial index"
	myGSI := anObject new
	! !

!CyberPhysicalSystem methodsFor: 'accessing' stamp: 'rbc 9/12/2011 18:06'!
mySSI
	"Get the socio spacial index"
	^mySSI
	! !

!CyberPhysicalSystem methodsFor: 'accessing' stamp: 'rbc 9/12/2011 18:06'!
mySSI: anObject
	"Create the socio spacial index"
	mySSI := anObject new
	! !

CyberPhysicalSystem subclass: #CyberspatialObject
	instanceVariableNames: 'myISI'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-CPS'!

!CyberspatialObject commentStamp: 'rbc 9/20/2011 20:11' prior: 0!
A cyberspatial object.!

!CyberspatialObject methodsFor: 'initialization' stamp: 'rbc 2/3/2012 17:20'!
createVSA
	"Pick up defaults from parent classes"
	super createVSA.
	"Add the CSO address object"
	self myISI: InfospacialIndex! !

!CyberspatialObject methodsFor: 'accessing' stamp: 'rbc 9/20/2011 19:52'!
myISI
	"Get the info spacial index"
	^myISI
	! !

!CyberspatialObject methodsFor: 'accessing' stamp: 'rbc 9/20/2011 19:52'!
myISI: anObject
	"Create the info spacial index"
	myISI := anObject new
	! !

VSA subclass: #HeaterOperation
	instanceVariableNames: 'myRoom myThermostat myHeater myHeatTransfer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Heater'!

!HeaterOperation commentStamp: 'rbc 7/31/2011 16:45' prior: 0!
A heater operation.

Copyright 2011 Robert Bruce Carleton. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY ROBERT BRUCE CARLETON ''AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ROBERT BRUCE CARLETON OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those of the
authors and should not be interpreted as representing official policies, either expressed
or implied, of Robert Bruce Carleton.
!

!HeaterOperation class methodsFor: 'instance creation' stamp: 'rbc 5/22/2011 12:39'!
new
	^super new startUp.
	! !

!HeaterOperation methodsFor: 'accessing' stamp: 'rbc 5/22/2011 14:03'!
desiredTemperature
	"Get the desired temperature"
	myThermostat desiredTemperature.! !

!HeaterOperation methodsFor: 'accessing' stamp: 'rbc 5/22/2011 14:08'!
desiredTemperature: aNumber
	"Set the desired temperature"
	myThermostat desiredTemperature: aNumber.
	myHeater setTemperature: myThermostat.
	self capability: aNumber.! !

!HeaterOperation methodsFor: 'events-triggering' stamp: 'rbc 5/22/2011 13:38'!
heatRoom
	"Heat the room"
	myHeater HeatRoom: myRoom.
	self actuality: myRoom temperature.! !

!HeaterOperation methodsFor: 'accessing' stamp: 'rbc 5/22/2011 15:20'!
maximumTemperature
	"Return the maximum temperature"
	self potentiality.! !

!HeaterOperation methodsFor: 'accessing' stamp: 'rbc 5/22/2011 15:19'!
maximumTemperature: aNumber
	"Set the maximum temperature"
	self potentiality: aNumber.! !

!HeaterOperation methodsFor: 'accessing' stamp: 'rbc 5/22/2011 12:32'!
myHeatTransfer
	"Get the heat transfer instance"
	^myHeatTransfer.
	! !

!HeaterOperation methodsFor: 'accessing' stamp: 'rbc 5/22/2011 12:38'!
myHeatTransfer: anObject
	"Create some heat transfer"
	myHeatTransfer := anObject new.
	! !

!HeaterOperation methodsFor: 'accessing' stamp: 'rbc 5/22/2011 12:33'!
myHeater
	"Return a heater instance"
	^myHeater.
	! !

!HeaterOperation methodsFor: 'accessing' stamp: 'rbc 5/22/2011 12:38'!
myHeater: anObject
	"Create a heater"
	myHeater := anObject new.
	! !

!HeaterOperation methodsFor: 'accessing' stamp: 'rbc 5/22/2011 12:34'!
myRoom
	"Get the room instance"
	^myRoom.
	! !

!HeaterOperation methodsFor: 'accessing' stamp: 'rbc 5/22/2011 12:38'!
myRoom: anObject
	"Create a room"
	myRoom := anObject new.
	! !

!HeaterOperation methodsFor: 'accessing' stamp: 'rbc 5/22/2011 12:36'!
myThermostat
	"Get the thermostat instance"
	^myThermostat.
	! !

!HeaterOperation methodsFor: 'accessing' stamp: 'rbc 5/22/2011 12:37'!
myThermostat: anObject
	"Create a thermostat"
	myThermostat := anObject new.
	! !

!HeaterOperation methodsFor: 'initialization' stamp: 'rbc 5/22/2011 15:21'!
startUp
	"Start the heater Operation"
	self myThermostat: Thermostat.
	myThermostat desiredTemperature: 70.
	self myHeatTransfer: HeatTransfer.
	myHeatTransfer transferRate: 2.
	self myHeater: Heater.
	myHeater setTemperature: myThermostat.
	self myRoom: Room.
	myHeater HeatRoom: myRoom.
	self actuality: myRoom temperature.
	self capability: myThermostat desiredTemperature.
	self maximumTemperature: 80.
	! !

!HeaterOperation methodsFor: 'events-triggering' stamp: 'rbc 5/22/2011 13:50'!
transferHeat
	"Remove heat from the room"
	myHeatTransfer transferHeat: myRoom.
	self actuality: myRoom temperature.! !

VSA subclass: #Institution
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!Institution commentStamp: 'rbc 8/20/2011 09:30' prior: 0!
A financial institution that may have many accounts.
!

Institution subclass: #Account
	instanceVariableNames: 'accountNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!Account commentStamp: 'rbc 8/20/2011 09:31' prior: 0!
A financial account that may hold many securities.!

!Account methodsFor: 'accessing' stamp: 'rbc 7/10/2011 12:42'!
accountName
	"Get the account name"
	^myExecutive vName
	! !

!Account methodsFor: 'accessing' stamp: 'rbc 7/10/2011 12:42'!
accountName: aString
	"Set the account name"
	myExecutive vName: aString
	! !

!Account methodsFor: 'accessing' stamp: 'rbc 7/10/2011 14:15'!
accountNumber
	"Get the account number"
	^accountNumber
	! !

!Account methodsFor: 'accessing' stamp: 'rbc 7/10/2011 14:15'!
accountNumber: aString
	"Set the account number"
	accountNumber := aString
	! !

Account subclass: #Cash
	instanceVariableNames: 'description stockPrice numberShares'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!Cash commentStamp: 'rbc 9/3/2011 12:36' prior: 0!
An Account subclass for storing cash.!

!Cash methodsFor: 'accessing' stamp: 'rbc 9/3/2011 11:53'!
balance
	"Get the balance"
	^self actuality! !

!Cash methodsFor: 'accessing' stamp: 'rbc 9/3/2011 12:02'!
deposit: aNumber
	"Deposit cash"
	actuality := actuality + aNumber
! !

!Cash methodsFor: 'accessing' stamp: 'rbc 9/3/2011 11:49'!
description
	"Get the cash description"
	^myExecutive vName! !

!Cash methodsFor: 'accessing' stamp: 'rbc 9/3/2011 11:48'!
description: aString
	"Set the cash description"
	myExecutive vName: aString! !

!Cash methodsFor: 'initialization' stamp: 'rbc 9/8/2011 14:41'!
initialBalance: aNumber
	"Setup the initial balance"
	self actuality: aNumber! !

!Cash methodsFor: 'accessing' stamp: 'rbc 9/3/2011 11:58'!
withdraw: aNumber
	"Withdraw cash"
	actuality > aNumber
	ifTrue: [actuality := actuality - aNumber. ^aNumber]
	ifFalse: [^nil]! !

Account subclass: #Stock
	instanceVariableNames: 'description stockPrice numberShares'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!Stock commentStamp: 'rbc 8/20/2011 09:31' prior: 0!
A stock.!

!Stock methodsFor: 'accessing' stamp: 'rbc 7/10/2011 14:18'!
actuality
	"Get the aggregate value of the position"
	^stockPrice * numberShares
	! !

!Stock methodsFor: 'accessing' stamp: 'rbc 6/7/2011 23:05'!
description
	"Get the stock description"
	^description! !

!Stock methodsFor: 'accessing' stamp: 'rbc 6/7/2011 23:05'!
description: aString
	"Set the stock description"
	description := aString! !

!Stock methodsFor: 'accessing' stamp: 'rbc 6/7/2011 23:16'!
numberShares
	"Get the number of shares"
	^numberShares! !

!Stock methodsFor: 'accessing' stamp: 'rbc 6/7/2011 23:16'!
numberShares: aNumber
	"Set the number of shares"
	numberShares := aNumber! !

!Stock methodsFor: 'accessing' stamp: 'rbc 7/10/2011 14:09'!
stockPrice
	"Get the stock price"
	^stockPrice! !

!Stock methodsFor: 'accessing' stamp: 'rbc 7/10/2011 14:16'!
stockPrice: aNumber
	"Set the stock price"
	stockPrice := aNumber! !

!Stock methodsFor: 'accessing' stamp: 'rbc 6/7/2011 22:56'!
ticker
	"Get the ticker"
	^myExecutive vName! !

!Stock methodsFor: 'accessing' stamp: 'rbc 6/7/2011 22:34'!
ticker: aString
	"Set the ticker"
	myExecutive vName: aString! !

Stock subclass: #StockOption
	instanceVariableNames: 'strikePrice marketPrice premium expiration numberContracts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockOption commentStamp: 'rbc 8/20/2011 09:31' prior: 0!
A stock option.!

!StockOption methodsFor: 'accessing' stamp: 'rbc 7/10/2011 14:24'!
actuality
	"Get the aggregate value of the position"
	^(numberContracts * 100) * marketPrice.
	! !

!StockOption methodsFor: 'calculations' stamp: 'rbc 7/5/2011 21:41'!
breakEven
	"Calculate and return the break even price for the option."
	
	self class == StockOptionCall ifTrue: [^ strikePrice + premium ].
	self class == StockOptionPut ifTrue: [^ strikePrice - premium ].
! !

!StockOption methodsFor: 'accessing' stamp: 'rbc 4/20/2009 06:19'!
expiration
	"Return the date of expiration"
	^ expiration
	! !

!StockOption methodsFor: 'accessing' stamp: 'rbc 7/10/2011 17:28'!
expiration: aDate
	"Set the date of expiration"
	expiration := aDate
	! !

!StockOption methodsFor: 'calculations' stamp: 'rbc 7/6/2011 07:50'!
intrinsicValue
	"Calculate and return the intrinsic value of the option"
	| sp |
	sp := self stockPrice.
	(self class == StockOptionCall) & (sp > strikePrice) ifTrue: [^ sp - strikePrice].
	(self class == StockOptionPut) & (sp < strikePrice) ifTrue: [^ strikePrice - sp].
	^0
	! !

!StockOption methodsFor: 'accessing' stamp: 'rbc 7/6/2011 01:57'!
marketPrice
	"Get the market price"
	^marketPrice.! !

!StockOption methodsFor: 'accessing' stamp: 'rbc 7/6/2011 01:57'!
marketPrice: aNumber
	"Set the market price"
	marketPrice := aNumber.! !

!StockOption methodsFor: 'accessing' stamp: 'rbc 7/10/2011 14:28'!
numberContracts
	"Get the number of contracts"
	^numberContracts! !

!StockOption methodsFor: 'accessing' stamp: 'rbc 7/10/2011 14:27'!
numberContracts: aNumber
	"Set the number of contracts"
	numberContracts := aNumber! !

!StockOption methodsFor: 'accessing' stamp: 'rbc 4/19/2009 21:40'!
premium
	"Get the premium"
	
	^ premium
! !

!StockOption methodsFor: 'accessing' stamp: 'rbc 4/19/2009 21:40'!
premium: aNumber
	"Set the premium"
	
	premium := aNumber
	! !

!StockOption methodsFor: 'accessing' stamp: 'rbc 4/19/2009 21:40'!
strikePrice
	"Get the striking price"
	
	^ strikePrice
	! !

!StockOption methodsFor: 'accessing' stamp: 'rbc 4/19/2009 21:41'!
strikePrice: aNumber
	"Set the striking price"
	
	strikePrice := aNumber.
	! !

!StockOption methodsFor: 'calculations' stamp: 'rbc 4/19/2009 21:08'!
timeValue
	"Calculate and return the time value of the option"
	
	| i |
	i := self intrinsicValue.
	premium > i ifTrue: [^ premium - i ].
	^ 0! !

StockOption subclass: #StockOptionCall
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockOptionCall commentStamp: 'rbc 8/20/2011 09:32' prior: 0!
A stock option call.!

StockOption subclass: #StockOptionPut
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Portfolio'!

!StockOptionPut commentStamp: 'rbc 8/20/2011 09:32' prior: 0!
A stock option put.!

!Institution methodsFor: 'accessing' stamp: 'rbc 7/10/2011 12:44'!
institutionName
	"Get the institution name."
	^myExecutive vName
	! !

!Institution methodsFor: 'accessing' stamp: 'rbc 7/10/2011 12:44'!
institutionName: aString
	"Set the institution name."
	myExecutive vName: aString
	! !

!VSA class methodsFor: 'instance creation' stamp: 'rbc 5/17/2011 20:35'!
new
	"Set up the VSA and it's systems"
	^super new createVSA! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 09:44'!
actuality
	"Get the actuality."
	^ actuality
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 09:44'!
actuality: aNumber
	"Set the actuality."
	actuality := aNumber
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 17:33'!
alert
	"Get an alert from the alerts queue"
	^alerts nextOrNil! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 13:33'!
alert: anObject
	"Add an alert to the alerts queue"
	alerts nextPut: anObject! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 09:45'!
alertVariance
	"Get the alert variance"
	^alertVariance.
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/30/2011 12:12'!
alertVariance: aNumber
	"Set the alert variance.  This specifies the threshold for creating an alert
	when actuality deviates from capability by aNumber.  The mark: method
	uses this parameter when it tests productivity."
	alertVariance := aNumber.
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 2/2/2012 18:48'!
alertsEnabled
	"Are alerts enabled"
	^alertsEnabled! !

!VSA methodsFor: 'accessing' stamp: 'rbc 2/2/2012 18:48'!
alertsEnabled: aBoolean
	"Enable alerts"
	alertsEnabled := aBoolean! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 13:40'!
capability
	"Get the capability."
	^ capability
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 13:40'!
capability: aNumber
	"Set the capability of the VSA."
	capability := aNumber
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 7/17/2011 16:09'!
costCenter
	"Get cost center boolean.  If it's not a profit center, it's a cost center"
	^costCenter
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 7/21/2011 23:15'!
costCenter: aBoolean
	"Set the cost center boolean. With a profit center, more is better.
	With a cost center less is better.  This boolean is used to determine
	how latency, performance and productivity are calculated.	See the
	end of chapter 11 in: 1972, Stafford Beer, Brain of the Firm"

	costCenter := aBoolean.
	myManager costCenter: aBoolean.
	! !

!VSA methodsFor: 'initialization' stamp: 'rbc 2/2/2012 18:51'!
createVSA
	"Create the VSA.  Create an alerts queue, set up the intial environment"
	self mySupervisor: VSASupervisor.
	self myExecutive: VSAExecutive.
	self myPlanner: VSAPlanner.
	self myManager: VSAManager.
	alerts := SharedQueue new.
	self actuality: 1.
	self capability: 1.
	self potentiality: 1.
	self alertVariance: 1.
	self runProcesses: false.
	self costCenter: false.
	self operationsCompare: true.
	self alertsEnabled: true! !

!VSA methodsFor: 'updating' stamp: 'rbc 9/24/2011 17:47'!
lastUpdate
	"Get the time of the last update"
	^lastUpdate! !

!VSA methodsFor: 'calculations' stamp: 'rbc 7/17/2011 21:39'!
latency
	"Calculate the latency."
	costCenter
	ifFalse: [^ capability / potentiality]
	ifTrue: [^ potentiality / capability]
	! !

!VSA methodsFor: 'events-triggering' stamp: 'rbc 2/2/2012 18:50'!
mark: anObject
	| a |
	"Mark an alert reporting threshold, generate an alert from the specified object if needed"
	alertsEnabled ifTrue: [
	actuality > (capability + alertVariance) ifTrue: [a := anObject new.
										a vName: myExecutive vName.
										a actuality: actuality.
										a capability: capability.
										self alert: a].
	actuality < (capability - alertVariance) ifTrue: [a := anObject new.	
										a vName: myExecutive vName.
										a actuality: actuality.
										a capability: capability.
										self alert: a].
									]
! !

!VSA methodsFor: 'message handling' stamp: 'rbc 12/23/2011 11:30'!
moveMessages
	"Move messages."
	| msg policy |

	"Move messages between operations."
	myManager supervisorMoveOpsMessages.
	
	"Send managers report to both planner and executive."
	[myManager outBoxSize > 0]
			whileTrue: [	msg := myManager outBox.
				myPlanner inBox: msg.
				myExecutive inBox: msg].
			
	"Send planners report to both manager and executive."
	[myPlanner outBoxSize > 0]
			whileTrue: [	msg := myPlanner outBox.
				myManager inBox: msg.
				myExecutive inBox: msg].
	
	"Send executive policy messages."
	[myExecutive outBoxSize > 0]
			whileTrue: [	msg := myExecutive outBox.
				myManager inBox: msg.
				myPlanner inBox: msg]

! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/17/2011 20:37'!
myExecutive
	"Return the executive"
	^myExecutive
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/17/2011 20:33'!
myExecutive: anObject
	"Create the executive"
	myExecutive := anObject new.
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/17/2011 20:40'!
myManager
	"Return the manager"
	^myManager
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/17/2011 20:39'!
myManager: anObject
	"Create the manager"
	myManager := anObject new.
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/17/2011 20:39'!
myPlanner
	"Return the planner"
	^myPlanner
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/17/2011 20:39'!
myPlanner: anObject
	"Create the planner"
	myPlanner := anObject new.
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 7/12/2011 17:19'!
mySupervisor
	"Return the supervisor"
	^mySupervisor
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 7/12/2011 17:19'!
mySupervisor: anObject
	"Create the supervisor"
	mySupervisor := anObject new.
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 11/29/2011 00:51'!
numberOfAlerts
	"Get the number of alerts"
	^alerts size! !

!VSA methodsFor: 'accessing' stamp: 'rbc 7/26/2011 20:22'!
operationsCompare
	"Do operations have comparable achievement numbers?"
	^operationsCompare
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 1/28/2012 14:11'!
operationsCompare: aBoolean
	"Do operations have comparable achievement numbers?
	This is used to control updates and manager calculations."
	operationsCompare := aBoolean.
	myManager operationsCompare: aBoolean
! !

!VSA methodsFor: 'calculations' stamp: 'rbc 7/17/2011 21:41'!
performance
	"Calculate performance"
	costCenter
	ifFalse: [^actuality / potentiality]
	ifTrue: [^potentiality / actuality]
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 09:46'!
potentiality
	"Get the potentiality."
	^ potentiality
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 09:47'!
potentiality: aNumber
	"Set the potentiality."
	potentiality := aNumber
	! !

!VSA methodsFor: 'calculations' stamp: 'rbc 7/17/2011 21:41'!
productivity
	"Calculate productivity."
	costCenter
	ifFalse: [^ actuality / capability]
	ifTrue: [^ capability / actuality]
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/30/2011 22:00'!
runProcesses
	"Get the intended process run state"
	^runProcesses.
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/30/2011 22:00'!
runProcesses: aBoolean
	"Set the intended process run state"
	runProcesses := aBoolean.
	! !

!VSA methodsFor: 'updating' stamp: 'rbc 9/24/2011 17:47'!
setLastUpdate
	"Set the time of the last update"
	lastUpdate := Date dateAndTimeNow! !

!VSA methodsFor: 'initialization' stamp: 'rbc 7/24/2011 18:00'!
startProcesses
	"Start VSA processes."
	self runProcesses: true.
	[[runProcesses] whileTrue:
		[self update.
		self mark: VSAAlert.
		self moveMessages.
		(Delay forSeconds: (myExecutive interval)) wait]] fork.
! !

!VSA methodsFor: 'updating' stamp: 'rbc 9/24/2011 17:48'!
update
	"Update actuality, capability and potentiality
	by extracting it from subordinate operations.
	This may not be desired if the subordinate
	operations do not have comparable achivement
	numbers.  Use operationsCompare to control updates.
	It is assumed that some other means of determining
	achivement will be provided, if subordinate operations do
	not provide comparable numbers."
	| b |
	b := myManager numberOfOperations.
	(((b = 1) | (b > 1)) & operationsCompare)
	ifTrue:
		[self actuality: myManager actualitySum.
		self capability: myManager capabilitySum.
		self potentiality: myManager potentialitySum.
		self setLastUpdate].
! !

VSA subclass: #VSAModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAModel commentStamp: 'rbc 3/1/2012 08:45' prior: 0!
A model VSA for use by the planner in determining system potentiality.
!

!VSAModel methodsFor: 'initialization' stamp: 'rbc 3/1/2012 08:43'!
createVSA
	"Create the model VSA.  Create an alerts queue, set up the intial environment.
	Note that we substitute the manager and alerts are disabled."

	self mySupervisor: VSASupervisor.
	self myExecutive: VSAExecutive.
	self myPlanner: VSAPlanner.
	self myManager: VSAManagerModel.
	alerts := SharedQueue new.
	self actuality: 1.
	self capability: 1.
	self potentiality: 1.
	self alertVariance: 1.
	self runProcesses: false.
	self costCenter: false.
	self operationsCompare: true.
	self alertsEnabled: false! !

Object subclass: #VSABackupRestore
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSABackupRestore commentStamp: 'rbc 7/27/2011 23:52' prior: 0!
Used to store and retrieve VSAs from disk.
!

!VSABackupRestore class methodsFor: 'objects from disk' stamp: 'rbc 5/25/2011 22:49'!
fetch: aString
	"Restore a VSA from file"
	| rr v |
	rr := ReferenceStream fileNamed: aString.
	v := rr next.
	rr close.
	^v! !

!VSABackupRestore class methodsFor: 'objects from disk' stamp: 'rbc 6/11/2011 16:32'!
save: anObject fileName: aString
	"Save an entire VSA object to disk"
	| rr |
	rr := ReferenceStream fileNamed: aString.
	rr nextPut: anObject.
	rr close.! !

Object subclass: #VSAExecutive
	instanceVariableNames: 'vName interval inBox outBox'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAExecutive commentStamp: 'rbc 7/28/2011 08:14' prior: 0!
A VSA executive class, or system 5.

-Policy decisions within the system as a whole
-Balancing demands from different parts of the system
-Steering the system as a whole
!

!VSAExecutive class methodsFor: 'instance creation' stamp: 'rbc 5/30/2011 19:01'!
new
	"Set up the executive"
	^super new createExecutive.
	! !

!VSAExecutive methodsFor: 'initialization' stamp: 'rbc 8/29/2011 07:22'!
createExecutive
	"Initialize the executive"
	self interval: 60.
	inBox := SharedQueue new.
	outBox := SharedQueue new
! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 8/29/2011 07:20'!
inBox
	"Read a message from the in box"
	^inBox nextOrNil! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 8/29/2011 07:21'!
inBox: anObject
	"Add a message object to the in box"
	inBox nextPut: anObject! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 5/30/2011 18:56'!
interval
	"Get the interval for for system activity, measured in seconds"
	^interval.
	! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 5/30/2011 18:56'!
interval: aNumber
	"Set the interval for for system activity, measured in seconds"
	interval := aNumber.
	! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 8/29/2011 07:21'!
outBox
	"Read a message from the out box"
	^outBox nextOrNil! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 8/29/2011 07:21'!
outBox: anObject
	"Add a message object to the out box"
	outBox nextPut: anObject
! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 12/23/2011 11:17'!
outBoxSize
	"Determine the size of the out box"
	^outBox size
! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 5/14/2011 23:00'!
vName
	"Get the VSA Name"
	^vName
	! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 5/14/2011 22:59'!
vName: aString
	"Set the VSA Name"
	vName := aString
	! !

Object subclass: #VSAManager
	instanceVariableNames: 'opsCollection inBox outBox costCenter operationsCompare'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAManager commentStamp: 'rbc 7/28/2011 08:13' prior: 0!
A VSA manager class, or system 3.

-Establishes the rules, resources and responsibilities of operations
-Communicates with Planner and Executive
-Sends audit messages to operations
-Accepts algedonic alerts from operations
!

!VSAManager class methodsFor: 'instance creation' stamp: 'rbc 5/30/2011 16:23'!
new
	"Set up the Manager"
	^super new createManager.
	! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:28'!
actualityMax
	"Calculate maximum actuality of all operations."
	| max n a |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [
		max := (opsCollection at: 1) actuality.
		opsCollection do: [:o | a := o actuality.
			max < a 
				ifTrue: [max := a]
			].
		^max]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:29'!
actualityMean
	"Calculate the mean actuality for all operations"
	| n s |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [s := self actualitySum. ^s / n]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:29'!
actualityMin
	"Calculate minimum actuality of all operations."
	| min n a |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [
		min := (opsCollection at: 1) actuality.
		opsCollection do: [:o | a := o actuality.
			min > a 
				ifTrue: [min := a]
			].
		^min]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:29'!
actualitySum
	"Calculate summed actuality of all operations."
	| s n |
	s := 0.
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [opsCollection do: [:a | s := s + a actuality]. ^s]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:29'!
capabilityMax
	"Calculate maximum capability of all operations."
	| max n c |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [
		max := (opsCollection at: 1) capability.
		opsCollection do: [:o | c := o capability.
			max < c 
				ifTrue: [max := c]
			].
		^max]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:29'!
capabilityMean
	"Calculate the mean capability for all operations"
	| n s |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [s := self capabilitySum. ^s / n]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:30'!
capabilityMin
	"Calculate minimum capability of all operations."
	| min n c |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [
		min := (opsCollection at: 1) capability.
		opsCollection do: [:o | c := o capability.
			min > c 
				ifTrue: [min := c]
			].
		^min]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:30'!
capabilitySum
	"Calculate summed capability of all operations."
	| s n |
	s := 0.
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [opsCollection do: [:a | s := s + a capability].	^s]
	ifFalse: [^nil]
	! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 7/17/2011 16:12'!
costCenter
	"Get cost center boolean.  If it's not a profit center, it's a cost center"
	^costCenter
	! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 7/17/2011 16:12'!
costCenter: aBoolean
	"Set cost center boolean.  If it's not a profit center, it's a cost center"
	costCenter := aBoolean
	! !

!VSAManager methodsFor: 'initialization' stamp: 'rbc 1/27/2012 18:11'!
createManager
	"Manager startup"
	inBox := SharedQueue new.
	outBox := SharedQueue new.
	costCenter := false.
	self operationsCompare: true! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 11/29/2011 00:05'!
createOp: anObject
	"Create an operation."
	| c |
	c := opsCollection isKindOf: OrderedCollection.
	c ifFalse: [opsCollection := OrderedCollection new].
	opsCollection add: anObject new
! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 11/29/2011 20:03'!
getAlerts
	"Retrieve and return alerts from subordinate operations"
	| a n c |
	c := OrderedCollection new.
	n := self numberOfOperations.
	n > 0
	ifTrue: [opsCollection do:
		[:o | [o numberOfAlerts > 0]
			whileTrue: [a := o alert. c add: a]
		]
	].
	^c! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 1/31/2010 13:58'!
getOp: aNumber
	"Get an operation."
	| op |
	op := opsCollection at: aNumber.
	^ op! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:55'!
inBox
	"Read a message from the in box"
	^inBox nextOrNil.
	
	! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:55'!
inBox: anObject
	"Add a message object to the in box"
	inBox nextPut: anObject
	! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:31'!
latency
	"Calculate aggregate latency of all operations."
	| c p n |
	c := self capabilitySum.
	p := self potentialitySum.
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [ costCenter
		ifFalse: [^c/p]
		ifTrue: [^p/c]
		]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 5/30/2011 21:44'!
numberOfOperations
	"Get the number of operations."
	| c |
	c := opsCollection isKindOf: OrderedCollection.
	c
	ifFalse: [^0]
	ifTrue: [^opsCollection size].
! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 1/27/2012 18:09'!
operationsCompare
	"Do operations have comparable achievement numbers?"
	^operationsCompare
	! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 1/27/2012 18:10'!
operationsCompare: aBoolean
	"Do operations have comparable achievement numbers?
	This is used to control calculations."
	operationsCompare := aBoolean
	! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:53'!
outBox
	"Read a message from the out box"
	^outBox nextOrNil.
	
	! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:53'!
outBox: anObject
	"Add a message object to the out box"
	outBox nextPut: anObject
	! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 12/23/2011 11:18'!
outBoxSize
	"Determine the size of the out box"
	^outBox size
! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:31'!
performance
	"Calculate aggregate performance of all operations."
	| a p n |
	a := self actualitySum.
	p := self potentialitySum.
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [ costCenter
		ifFalse: [^ a / p]
		ifTrue: [^ p / a]
		]
	ifFalse: [^nil]
! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:31'!
potentialityMax
	"Calculate maximum potentiality of all operations."
	| max n p |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [
		max := (opsCollection at: 1) potentiality.
		opsCollection do: [:o | p := o potentiality.
			max < p 
				ifTrue: [max := p]
			].
		^max]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:31'!
potentialityMean
	"Calculate the mean potentiality for all operations"
	| n s |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [s := self potentialitySum. ^s / n]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:32'!
potentialityMin
	"Calculate minimum potentiality of all operations."
	| min n p |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [
		min := (opsCollection at: 1) potentiality.
		opsCollection do: [:o | p := o potentiality.
			min > p 
				ifTrue: [min := p]
			].
		^min]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:32'!
potentialitySum
	"Calculate summed potentiality of all operations."
	| p n |
	p := 0.
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [opsCollection do: [:a | p := p + a potentiality]. ^p]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:32'!
productivity
	"Calculate aggregate productivity of all operations."
	| a c n |
	a := self actualitySum.
	c := self capabilitySum.
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [ costCenter
		ifFalse: [^a/c]
		ifTrue: [^c/a]
		]
	ifFalse: [^nil]
! !

!VSAManager methodsFor: 'message handling' stamp: 'rbc 2/29/2012 17:20'!
sendCopyOpsCollection: aMessageClass
	"Send a copy of the ops collection to planner and executive. If the ops collection
	contains a large hierarchy of VSAs, running this method can get expensive."
	| c m |
	c := opsCollection veryDeepCopy.
	m := aMessageClass new.
	m message: c.
	self outBox: m
! !

!VSAManager methodsFor: 'message handling' stamp: 'rbc 11/29/2011 00:19'!
sendManagerReport: anObject
	"Send manager report to planner"
	| a c l m p report |
	a := self actualitySum.
	c := self capabilitySum.
	l := self latency.
	m := self getAlerts.
	p := self performance.
	report := anObject new.
	report actuality: a.
	report capability: c.
	report latency: l.
	report performance: p.
	report message: m.
	self outBox: report.
	 ! !

!VSAManager methodsFor: 'message handling' stamp: 'rbc 7/24/2011 09:12'!
sendOpsMessage: aMessage
	"Send a message to an operation"
	| to op |
	to := aMessage to.
	op := self getOp: to.
	op mySupervisor inBox: aMessage! !

!VSAManager methodsFor: 'message handling' stamp: 'rbc 12/17/2011 00:59'!
supervisorMoveOpsMessages
	"Move ops messages from one operation to another"
	| n msg |
	n := self numberOfOperations.
	n > 0
	ifTrue: [
		opsCollection do: [ :op | [op mySupervisor outBoxSize > 0]
			whileTrue: [msg := op mySupervisor outBox.
						self sendOpsMessage: msg]
		]
	]
! !

VSAManager subclass: #VSAManagerModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAManagerModel commentStamp: 'rbc 3/1/2012 08:44' prior: 0!
A model manager for use by planner to determine system potentiality.
!

!VSAManagerModel methodsFor: 'accessing' stamp: 'rbc 3/1/2012 08:09'!
opsCollection
	"Get the modified ops collection"
	^opsCollection! !

!VSAManagerModel methodsFor: 'accessing' stamp: 'rbc 3/1/2012 08:46'!
opsCollection: anOpsCollection
	"Insert an existing copy of the ops collection"
	opsCollection := anOpsCollection! !

Object subclass: #VSAMessage
	instanceVariableNames: 'timeStamp message vName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAMessage commentStamp: 'rbc 9/1/2011 22:08' prior: 0!
A stock message, usually subclassed.
!

VSAMessage subclass: #VSAAlert
	instanceVariableNames: 'actuality capability'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAAlert commentStamp: 'rbc 11/30/2011 19:52' prior: 0!
A class for alerts sent by an Operation to Manager.  The life cycle of an alert consists of 3 major steps.

1) Alerts are generated by VSA>>mark: using the specified class, usually VSAAlert.  They are stored in the alerts SharedQueue inside the operation.  The threshold for generating an alert is specified by the VSA alertVarience instance variable.

2) VSAManager>>getAlerts is called by the VSAManager>>sendManagerReport: method.  It retrieves and returns all alerts, for all operations in its opsCollection.  This also has the effect of clearing the alerts from the subordinate operations alerts queue.  The alerts collection is stored in the VSAManagerReport message variable.  VSAManager>>sendManagerReport finishes by placing the VSAManagerReport in the VSAManager outBox SharedQueue.

3) The VSAManagerReport is retrived from the VSAManager outBox by VSA>>moveMessages.  It is copied to both the inBox for myExecutive (a VSAExecutive) and myPlanner (a VSAPlanner).

Once myExecutive and myPlanner have the VSAManagerReport, they can use the achievement data and alerts in the report for application specific purposes.
!

!VSAAlert methodsFor: 'accessing' stamp: 'rbc 6/1/2011 21:58'!
actuality
	"Get the actuality."
	^actuality! !

!VSAAlert methodsFor: 'accessing' stamp: 'rbc 6/1/2011 21:50'!
actuality: aNumber
	"Set the actuality"
	actuality := aNumber! !

!VSAAlert methodsFor: 'accessing' stamp: 'rbc 6/1/2011 21:50'!
capability
	"Get the capability"
	^capability! !

!VSAAlert methodsFor: 'accessing' stamp: 'rbc 6/1/2011 21:50'!
capability: aNumber
	"Set the capability"
	capability := aNumber! !

VSAMessage subclass: #VSAManagerReport
	instanceVariableNames: 'actuality capability latency performance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAManagerReport commentStamp: 'rbc 11/30/2011 19:54' prior: 0!
A report sent to Planner and Executive.  It contains achievement data and alerts for subordinate operations.
!

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 9/8/2011 10:27'!
actuality
	"Get the actuality"
	^actuality! !

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 9/8/2011 10:27'!
actuality: aNumber
	"Set the actuality"
	actuality := aNumber! !

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 9/8/2011 10:29'!
capability
	"Get the capability"
	^capability! !

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 9/8/2011 10:28'!
capability: aNumber
	"Set the capability"
	capability := aNumber! !

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 5/30/2011 17:39'!
latency
	"Get the latency"
	^latency.
	! !

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 5/30/2011 17:39'!
latency: aNumber
	"Set the latency"
	latency := aNumber.
	! !

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 5/30/2011 17:40'!
performance
	"Get the performance"
	^performance.
	! !

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 5/30/2011 17:40'!
performance: aNumber
	"Set the performance"
	performance := aNumber.
	! !

!VSAMessage class methodsFor: 'instance creation' stamp: 'rbc 9/1/2011 21:54'!
new
	^super new setTimeStamp.! !

!VSAMessage methodsFor: 'accessing' stamp: 'rbc 9/1/2011 21:54'!
message
	"Get the message"
	^message! !

!VSAMessage methodsFor: 'accessing' stamp: 'rbc 9/1/2011 21:54'!
message: anObject
	"Set the message"
	message := anObject! !

!VSAMessage methodsFor: 'accessing' stamp: 'rbc 9/1/2011 21:54'!
setTimeStamp
	"Set the time stamp"
	timeStamp := Date dateAndTimeNow! !

!VSAMessage methodsFor: 'accessing' stamp: 'rbc 9/1/2011 21:54'!
timeStamp
	"Get the time stamp"
	^timeStamp! !

!VSAMessage methodsFor: 'accessing' stamp: 'rbc 11/24/2011 10:00'!
vName
	"Get the name of the sending VSA"
	^vName! !

!VSAMessage methodsFor: 'accessing' stamp: 'rbc 11/24/2011 10:00'!
vName: aString
	"Set the name of the sending VSA"
	vName := aString! !

VSAMessage subclass: #VSAOpsCopy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAOpsCopy commentStamp: 'rbc 2/29/2012 12:46' prior: 0!
A message containing a copy of the VSA ops collection.
!

VSAMessage subclass: #VSAOpsMessage
	instanceVariableNames: 'to'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAOpsMessage commentStamp: 'rbc 11/30/2011 20:09' prior: 0!
A message, sent between operations.  It is used for operations needing to send objects between one another. 

Delivery of ops messages is done by VSAManager>>supervisorMoveOpsMessages, usually called by VSA>>moveMessages.  Messages are sent via an operations mySupervisor, typically a VSASupervisor.  

VSASupervisor can be subclassed to add application specific anti-oscillation checks to inter-operation messages.  mySupervisor is created by VSA>>createVSA.  Subclassing the VSA allows substituting the class used for mySupervisor, along with myExecutive and myPlanner.
!

!VSAOpsMessage methodsFor: 'accessing' stamp: 'rbc 7/14/2011 20:27'!
to
	"Get the operation number that the message is addressed to"
	^to! !

!VSAOpsMessage methodsFor: 'accessing' stamp: 'rbc 7/14/2011 20:27'!
to: aNumber
	"Set the operation number that the message is addressed to"
	to := aNumber! !

VSAMessage subclass: #VSAPlannerMessage
	instanceVariableNames: 'potentiality'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAPlannerMessage commentStamp: 'rbc 11/30/2011 21:17' prior: 0!
A class for sending planning messages from VSAPlanner or its subclass. It is sent from myPlanner to myExecutive (VSAExecutive) and myManager (VSAManager) by VSA>>moveMessages. Planner potentiality calculations are application specific. The message instance variable can be used to ship any kind of object.
!

!VSAPlannerMessage methodsFor: 'accessing' stamp: 'rbc 9/8/2011 10:11'!
potentiality
	"Get the potentiality"
	^potentiality! !

!VSAPlannerMessage methodsFor: 'accessing' stamp: 'rbc 9/8/2011 10:10'!
potentiality: aNumber
	"Set the potentiality"
	potentiality := aNumber! !

VSAMessage subclass: #VSAPolicyMessage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAPolicyMessage commentStamp: 'rbc 11/30/2011 21:19' prior: 0!
A policy message, sent by myExecutive.  It is sent to myPlanner (VSAPlanner) and myManager (VSAManager) by VSA>>moveMessages. The message instance variable can be used to ship any kind of object.
!

Object subclass: #VSAPlanner
	instanceVariableNames: 'inBox outBox policyMsgs managerReports opsCopy modelVSA'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSAPlanner commentStamp: 'rbc 7/28/2011 08:20' prior: 0!
A VSA planner class, system 4.

-Responsible for looking outwards to the environment
-Communicates with Manager and Executive
-Determines system potentiality!

!VSAPlanner class methodsFor: 'instance creation' stamp: 'rbc 5/30/2011 16:23'!
new
	"Set up the Planner"
	^super new createPlanner.
	! !

!VSAPlanner methodsFor: 'initialization' stamp: 'rbc 3/4/2012 11:18'!
createPlanner
	"Planner startup"
	inBox := SharedQueue new.
	outBox := SharedQueue new
! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 3/4/2012 12:56'!
getManagerReport: aNumber
	"Get a manager report."
	| r |
	r := managerReports at: aNumber.
	^ r! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 3/4/2012 12:56'!
getPolicyMsg: aNumber
	"Get a policy message."
	| m |
	m := policyMsgs at: aNumber.
	^ m! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:55'!
inBox
	"Read a message from the in box"
	^inBox nextOrNil.
	
	! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:55'!
inBox: anObject
	"Add a message object to the in box"
	inBox nextPut: anObject
	! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 3/4/2012 12:36'!
inBoxSize
	"Determine the size of the inbox"
	^inBox size
! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 3/4/2012 12:45'!
opsCopy
	"Get a copy of the ops collection"
	^opsCopy! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 3/4/2012 12:44'!
opsCopy: aCollection
	"Save a copy of the ops collection"
	opsCopy := aCollection! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:54'!
outBox
	"Read a message from the out box"
	^outBox nextOrNil.
	
	! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:55'!
outBox: anObject
	"Add a message object to the out box"
	outBox nextPut: anObject
	! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 12/23/2011 11:18'!
outBoxSize
	"Determine the size of the out box"
	^outBox size
! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 3/4/2012 12:51'!
saveManagerReport: anObject
	"Add a message object to the in manager report store"
	
	| t |
	t := managerReports isKindOf: OrderedCollection.
	t ifFalse: [managerReports := OrderedCollection new].
	managerReports add: anObject! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 3/4/2012 12:51'!
savePolicyMsg: anObject
	"Add a message object to the in policy store"
	
	| t |
	t := policyMsgs isKindOf: OrderedCollection.
	t ifFalse: [policyMsgs := OrderedCollection new].
	policyMsgs add: anObject! !

!VSAPlanner methodsFor: 'message handling' stamp: 'rbc 3/4/2012 12:51'!
sortMail
	"Sort the inbound mail"
	| msg t |
	[self inBoxSize > 0]
			whileTrue: [	msg := self inBox.
						(msg isKindOf: VSAPolicyMessage)
							ifTrue: [self savePolicyMsg: msg].
						(msg isKindOf: VSAManagerReport)
							ifTrue: [self saveManagerReport: msg].
						(msg isKindOf: VSAOpsCopy)
							ifTrue: [self opsCopy: msg].
	]
! !

Object subclass: #VSASupervisor
	instanceVariableNames: 'inBox outBox'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA-Core'!

!VSASupervisor commentStamp: 'rbc 11/30/2011 21:22' prior: 0!
A supervisor (system 2), paired with an operation (system 1).

-Provides conduits for communications between multiple operations
-Provides Manager with conduits to monitor and coordinate system operation activities
-Communicates with paired Operation, other supervisors and Manager

VSASupervisor can be subclassed to add application specific anti-oscillation checks to inter-operation messages.  mySupervisor is created by VSA>>createVSA.  Subclassing the VSA allows substituting the class used for mySupervisor, along with myExecutive and myPlanner.
!

!VSASupervisor class methodsFor: 'instance creation' stamp: 'rbc 7/12/2011 17:21'!
new
	"Set up the Supervisor"
	^super new createSupervisor.
	! !

!VSASupervisor methodsFor: 'initialization' stamp: 'rbc 7/12/2011 17:21'!
createSupervisor
	"Supervisor startup"
	inBox := SharedQueue new.
	outBox := SharedQueue new.
! !

!VSASupervisor methodsFor: 'accessing' stamp: 'rbc 7/12/2011 17:20'!
inBox
	"Read a message from the in box"
	^inBox nextOrNil.
	
	! !

!VSASupervisor methodsFor: 'accessing' stamp: 'rbc 7/12/2011 17:20'!
inBox: anObject
	"Add a message object to the in box"
	inBox nextPut: anObject
	! !

!VSASupervisor methodsFor: 'accessing' stamp: 'rbc 7/12/2011 17:20'!
outBox
	"Read a message from the out box"
	^outBox nextOrNil.
	
	! !

!VSASupervisor methodsFor: 'accessing' stamp: 'rbc 7/12/2011 17:20'!
outBox: anObject
	"Add a message object to the out box"
	outBox nextPut: anObject
	! !

!VSASupervisor methodsFor: 'accessing' stamp: 'rbc 12/17/2011 00:59'!
outBoxSize
	"Determine the size of the out box"
	^outBox size
! !
